@@ -1,13 +1,18 @@
 /**************************************************************************/
 /*                                                                        */
-/*  The Why/Caduceus/Krakatoa tool suite for program certification        */
-/*  Copyright (C) 2002-2006                                               */
+/*  The Why platform for program certification                            */
+/*  Copyright (C) 2002-2008                                               */
+/*    Romain BARDOU                                                       */
 /*    Jean-Franois COUCHOT                                               */
 /*    Mehdi DOGGUY                                                        */
 /*    Jean-Christophe FILLITRE                                           */
 /*    Thierry HUBERT                                                      */
 /*    Claude MARCH                                                       */
 /*    Yannick MOY                                                         */
+/*    Christine PAULIN                                                    */
+/*    Yann RGIS-GIANAS                                                   */
+/*    Nicolas ROUSSET                                                     */
+/*    Xavier URBAIN                                                       */
 /*                                                                        */
 /*  This software is free software; you can redistribute it and/or        */
 /*  modify it under the terms of the GNU General Public                   */
@@ -22,9 +27,9 @@
 /*                                                                        */
 /**************************************************************************/
 
-/* vrifie que t[0..n-1] ne contient que des 0 */
+/* check that t[0..n-1] only contains 0 */
 
-/*@ requires \valid_range(t,0,n) 
+/*@ requires n >= 0 && \valid_range(t,0,n) 
     ensures \result <=> \forall int i; 0<=i<n => t[i]==0 */
 int all_zeros(int t[], int n) {
   /*@ invariant n <= \old(n) && \forall int i; n<=i<\old(n) => t[i]==0
@@ -33,16 +38,7 @@ int all_zeros(int t[], int n) {
   return n < 0;
 }
 
-/*@ requires \valid_range(t,0,n) 
-    ensures \result <=> \forall int i; 0<=i<n => t[i]==0 */
-int all_zeros_1(int t[], int n) {
-  int k = 0;
-  /*@ invariant 0 <= k <= n && \forall int i; 0<=i<k => t[i]==0 variant n-k */
-  while (k<n && !t[k++]);
-  return k == n;
-}
-
-/*@ requires \valid_range(t,0,n) 
+/*@ requires n >= 0 && \valid_range(t,0,n) 
     ensures \result <=> \forall int i; 0<=i<n => t[i]==0 */
 int all_zeros_0(int t[], int n) {
   int k;
