@@ -1,13 +1,18 @@
 /**************************************************************************/
 /*                                                                        */
-/*  The Why/Caduceus/Krakatoa tool suite for program certification        */
-/*  Copyright (C) 2002-2006                                               */
+/*  The Why platform for program certification                            */
+/*  Copyright (C) 2002-2008                                               */
+/*    Romain BARDOU                                                       */
 /*    Jean-Franois COUCHOT                                               */
 /*    Mehdi DOGGUY                                                        */
 /*    Jean-Christophe FILLITRE                                           */
 /*    Thierry HUBERT                                                      */
 /*    Claude MARCH                                                       */
 /*    Yannick MOY                                                         */
+/*    Christine PAULIN                                                    */
+/*    Yann RGIS-GIANAS                                                   */
+/*    Nicolas ROUSSET                                                     */
+/*    Xavier URBAIN                                                       */
 /*                                                                        */
 /*  This software is free software; you can redistribute it and/or        */
 /*  modify it under the terms of the GNU General Public                   */
@@ -22,6 +27,8 @@
 /*                                                                        */
 /**************************************************************************/
 
+/*@ axiom A : \forall double t[]; \forall int i; \valid(t+i) => t[i]>0.0 => t[i]>=0.0 */
+
 typedef struct U { int t2[5]; int t2bis[5]; int *p2; } las2;
 typedef struct V { int t1[5]; int t1bis[5]; int *p1; las2 * pp; } las;
 las u,v,w;
@@ -37,3 +44,23 @@ las2 y2[10];
 
 /*@ requires \valid(x) */
 void f(struct U *x) { x->t2[0] = 1; *u.p1 = 1; *z.p2 = 2; }
+
+int ff(double a,double b)
+{
+int x;
+x=(a<b);
+return x;
+}
+
+//@ ensures \result < a
+double f2(double a) { return (a - 1.2e-3); }
+ 
+/*@ requires 0.0 < x <= 1.0
+*/
+void f3(double x)
+{
+/* ... */
+x = x * 2.0;
+/* ... */
+}
+
