@@ -1,3 +1,31 @@
+/**************************************************************************/
+/*                                                                        */
+/*  The Why platform for program certification                            */
+/*  Copyright (C) 2002-2008                                               */
+/*    Romain BARDOU                                                       */
+/*    Jean-Franois COUCHOT                                               */
+/*    Mehdi DOGGUY                                                        */
+/*    Jean-Christophe FILLITRE                                           */
+/*    Thierry HUBERT                                                      */
+/*    Claude MARCH                                                       */
+/*    Yannick MOY                                                         */
+/*    Christine PAULIN                                                    */
+/*    Yann RGIS-GIANAS                                                   */
+/*    Nicolas ROUSSET                                                     */
+/*    Xavier URBAIN                                                       */
+/*                                                                        */
+/*  This software is free software; you can redistribute it and/or        */
+/*  modify it under the terms of the GNU General Public                   */
+/*  License version 2, as published by the Free Software Foundation.      */
+/*                                                                        */
+/*  This software is distributed in the hope that it will be useful,      */
+/*  but WITHOUT ANY WARRANTY; without even the implied warranty of        */
+/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  */
+/*                                                                        */
+/*  See the GNU General Public License version 2 for more details         */
+/*  (enclosed in the file GPL).                                           */
+/*                                                                        */
+/**************************************************************************/
 
 /* Priority queues */
 
@@ -173,10 +201,24 @@ int max() {
   @*/
 int pop() {
   int res = heap[0];
-  int i = 0;
-  heap[0] = heap[--size];
-  while (i < size) {
-    
+  if (--size) {
+    int v = heap[size]; // value to insert
+    int i = 0;          // candidate position
+    /*@ invariant
+      @   0 <= i <= size // TODO: complete invariant
+      @ loop_assigns
+      @   heap[..]
+      @ variant
+      @   size - i
+      @*/
+    while (i < size) {
+      int j = 2*i+1;
+      if (j < size-1 && heap[j] < heap[j+1]) j++;
+      if (v >= heap[j]) break;
+      heap[i] = heap[j];
+      i = j;
+    }
+    heap[i] = v;
   }
   return res;
 }
