@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: ceffect.ml,v 1.163 2007-06-28 09:07:44 marche Exp $ i*)
+(*i $Id: ceffect.ml,v 1.168 2008/05/28 14:53:34 marche Exp $ i*)
 
 open Cast
 open Cnorm
@@ -123,17 +128,6 @@ let union = ZoneSet.union
 
 (* static variables *)
 
-(**
-let memorycell_name (ty : Info.why_type) =
-  match ty with
-    | Pointer z -> "pointer"
-    | Addr z -> assert false
-    | Info.Int -> "int" 
-    | Info.Float ->  "real"
-    | Unit -> assert false
-    | Why_Logic s ->  s
-    | Memory(t,z) -> assert false
-**)
 
 let add_var v (ty : Info.why_type) s =
   let info = declare_heap_var v v.var_unique_name in
@@ -190,10 +184,23 @@ type effect =
     }
 
 let ef_empty = { reads = empty; assigns = empty ; 
-		 reads_var = HeapVarSet.empty ; assigns_var = HeapVarSet.empty;
+		 reads_var = HeapVarSet.empty ; 
+		 assigns_var = HeapVarSet.empty;
 		 reads_under_pointer = HeapVarSet.empty;
 		 assigns_under_pointer = HeapVarSet.empty; }
 
+(*
+let merge_var_map m1 m2 =
+  HeapVarMap.fold
+    (fun v labs acc ->
+       try
+	 let l = HeapVarMap.find v m2 in
+	 HeapVarMap.add v (LabelSet.union labs l) acc
+       with Not_found ->
+	   HeapVarMap.add v labs acc)
+    m1 m2
+*)
+
 let ef_union e1 e2 = 
   { reads = union e1.reads e2.reads;
     assigns = union e1.assigns e2.assigns ;
@@ -532,8 +539,9 @@ let spec sp =
 	   (ef_option predicate sp.ensures))
 	(ef_option variant sp.decreases))
     (ef_option 
-       (List.fold_left
-	  (fun acc l -> ef_union acc (assign_location l)) ef_empty)
+       (fun (_,l) -> 
+	 List.fold_left
+	   (fun acc l -> ef_union acc (assign_location l)) ef_empty l)
        sp.Clogic.assigns)
 
 open Cast
