@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -232,7 +237,6 @@ let get_fresh_name n = unique_name n
 
 let types = Hashtbl.create 17
 let add_type x = Hashtbl.add types x ()
-(*let () = add_type "real"*)
 let mem_type = Hashtbl.mem types
 let find_type = Hashtbl.find types
 let iter_types f = Hashtbl.iter (fun s _ -> f s) types
@@ -289,6 +293,13 @@ let declare_int_size ((s,i) as is) =
 let all_int_sizes () = SI.elements !int_sizes
 let is_int_type s = StringSet.mem s !int_types
 
+let real_types =
+  StringSet.add "single" 
+    (StringSet.add "quad" 
+       (StringSet.singleton "double"))
+
+let is_real_type s = StringSet.mem s real_types
+
 let int_size = function
   | Char -> char_size
   | Short -> short_size
@@ -315,11 +326,11 @@ let rec type_type_why ?name ty zone_is_var =
     | Tenum _ when not Coptions.enum_check -> Int
     | Tint ik -> Why_Logic (int_type_for ik)
     | Tenum e -> Why_Logic (enum_type_for e)
-    | Tfloat _ when not Coptions.floats -> Why_Logic "real"
+    | Tfloat _ when not Coptions.floats -> Real
     | Tfloat Float -> Why_Logic "single"
     | Tfloat Double -> Why_Logic "double"
     | Tfloat LongDouble -> Why_Logic "quad"
-    | Tfloat Real -> Why_Logic "real"
+    | Tfloat Ctypes.Real -> Real
     | Tarray (_,ty,_)  
     | Tpointer (_,ty) -> 
 	begin match ty.ctype_node with 
