@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -74,7 +79,7 @@ let spec s =
   begin
   match s.assigns with
     | None -> []
-    | Some l -> List.fold_left (fun acc t -> (term t) @acc)  [] l
+    | Some (_, l) -> List.fold_left (fun acc t -> (term t) @acc)  [] l
   end @
   begin
     match s.ensures with
@@ -96,7 +101,7 @@ let loop_annot la =
   begin
   match la.loop_assigns with
     | None -> []
-    | Some l -> List.fold_left (fun acc t -> (term t) @acc)  [] l
+    | Some (_,l) -> List.fold_left (fun acc t -> (term t) @acc)  [] l
   end @
   begin
     match la.variant with
