@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: cinterp.ml,v 1.247 2007-07-03 08:32:31 hubert Exp $ i*)
+(*i $Id: cinterp.ml,v 1.259 2008/05/28 13:31:05 marche Exp $ i*)
 
 open Format
 open Coptions
@@ -35,6 +40,75 @@ open Ctypes
 open Cseparation
 open Pp
 
+(* locs table *)
+
+type kind =
+  | ArithOverflow
+  | DownCast
+  | IndexBounds
+  | PointerDeref
+  | UserCall
+  | DivByZero
+  | Pack
+  | Unpack
+
+let locs_table = Hashtbl.create 97
+let name_counter = ref 0
+
+let abs_fname f =
+  if Filename.is_relative f then
+    Filename.concat (Unix.getcwd ()) f 
+  else f
+
+let reg_loc ?id ?oldid ?kind ?name ?beh (b,e) =  
+  let id,oldid = match id,oldid with
+    | None,_ ->  
+	incr name_counter;
+	"CADUCEUS_" ^ string_of_int !name_counter, oldid
+    | Some n, None -> n,Some n
+    | Some n, Some o -> n, Some o
+  in
+  let (name,f,l,b,e) = 
+      let f = abs_fname b.Lexing.pos_fname in
+      let l = b.Lexing.pos_lnum in
+      let fc = b.Lexing.pos_cnum - b.Lexing.pos_bol in
+      let lc = e.Lexing.pos_cnum - b.Lexing.pos_bol in
+      (name,f,l,fc,lc)
+  in
+  Coptions.lprintf "recording location for id '%s'@." id;
+  Hashtbl.replace locs_table id (kind,name,beh,f,l,b,e);
+  id
+    
+let print_kind fmt k =
+  fprintf fmt "%s"
+    (match k with
+       | Pack -> "Pack"
+       | Unpack -> "Unpack"
+       | DivByZero -> "DivByZero"
+       | UserCall -> "UserCall"
+       | PointerDeref -> "PointerDeref"
+       | IndexBounds -> "IndexBounds"
+       | DownCast -> "DownCast"
+       | ArithOverflow -> "ArithOverflow"
+    )
+
+let print_locs fmt =
+  Hashtbl.iter 
+    (fun id (kind,name,beh,f,l,b,e) ->
+       fprintf fmt "[%s]@\n" id;
+       Option_misc.iter
+	 (fun k -> fprintf fmt "kind = %a@\n" print_kind k) kind;
+       Option_misc.iter
+	 (fun n -> fprintf fmt "name = \"%s\"@\n" n) name;
+       Option_misc.iter
+	 (fun b -> fprintf fmt "behavior = \"%s\"@\n" b) beh;
+       fprintf fmt "file = \"%s\"@\n" f;
+       fprintf fmt "line = %d@\n" l;
+       fprintf fmt "begin = %d@\n" b;
+       fprintf fmt "end = %d@\n@\n" e)
+    locs_table
+
+(* end locs table *)
 
 let print_effects2 fmt l =
   fprintf fmt "@[%a@]"
@@ -193,6 +267,7 @@ let zoned_name (f : string) (ty : Info.why_type) =
     | Pointer z -> f ^ "_" ^ (found_repr ~quote_var:false z)
     | Addr _ 
     | Info.Int -> assert false
+    | Info.Real -> assert false
     | Unit -> assert false
     | Why_Logic s ->  assert false
     | Memory(t,z) -> assert false
@@ -298,8 +373,8 @@ let rec interp_term label old_label t =
 	assert false
     | NTat (t, l) -> 
 	interp_term (Some l) old_label t
-    | NTif (_, _, _) -> 
-	unsupported t.nterm_loc "logic if-then-else"
+    | NTif (t1, t2, t3) -> 
+	LApp ("ite", [interp_boolean_term label old_label t1; f t2; f t3])
     | NTarrow (t,z, field) -> 
 	let te = f t in
 	let var = zoned_name field.var_unique_name (Cnorm.type_why_for_term t)
@@ -315,7 +390,10 @@ let rec interp_term label old_label t =
 	interp_term_un_op t1.nterm_type op (f t1)
     | NTunop (Uplus, t1) ->
 	(f t1)
-    | NTunop (Unot, t1) -> LApp ("non_int", [f t])
+    | NTunop (Unot, t1) -> 
+	LApp ("ite", 
+	      [interp_boolean_term label old_label t1; 
+	       LConst (Prim_int "0"); LConst (Prim_int "1")])
     | NTunop (Ufloat_of_int, t1) ->
 	let e = LApp ("real_of_int", [f t1]) in
 	begin match t.nterm_type.ctype_node with
@@ -381,8 +459,8 @@ let rec interp_term label old_label t =
 	  v.logic_heap_zone [] in
 	let targs = List.map f tl in
 	let targs = HeapVarSet.fold 
-		 (fun x acc -> (interp_var label (heap_var_name x)) :: acc) 
-		 v.logic_heap_args targs in
+	  (fun x acc -> (interp_var label (heap_var_name x)) :: acc) 
+	  v.logic_heap_args targs in
 	let targs = List.fold_right 
 	  (fun (z,s,_) l -> (interp_var label (zoned_name s (Pointer z)))::l)
 	  reads targs 
@@ -432,6 +510,20 @@ and interp_term_address  label old_label e = match e.nterm_node with
   | _ -> 
       assert false (* not a left value *)
 
+and interp_boolean_term label old_label t = 
+  (* t <> 0 *)
+  let cmp,zero = match t.nterm_type.Ctypes.ctype_node with
+    | Tenum _ | Tint _ -> 
+	"neq_int_bool", LConst (Prim_int "0")
+    | Tfloat fk -> 
+	assert false (* TODO *)
+    | Tarray _ | Tpointer _ -> 
+	assert false (* TODO *)
+    | _ -> 
+	assert false
+  in
+  LApp (cmp, [interp_term label old_label t; zero])
+
 let has_prefix p s = 
   let n = String.length p in String.length s >= n && String.sub s 0 n = p
 
@@ -548,12 +640,22 @@ let rec interp_predicate label old_label p =
     | NPbound_separated (t1,t2,t3,t4) ->
 	LPred("bound_separated",[ft t1;ft t2;ft t3;ft t4])
 
+(*
 let interp_predicate label old_label p = 
   let w = interp_predicate label old_label p in
   if p.npred_loc = Loc.dummy_position then
     w
   else
     LNamed ("\"" ^ String.escaped (Loc.string p.npred_loc) ^ "\"", w)
+*)
+
+let named_predicate loc = function
+  | LNamed _ as a -> a
+  | a -> let n = reg_loc loc in LNamed (n,a)
+
+let interp_predicate label oldlabel a =
+  let a' = interp_predicate label oldlabel a in
+  named_predicate a.npred_loc a'
 
 let interp_predicate_opt label old_label pred =
   match pred with
@@ -595,6 +697,9 @@ let interp_float_conversion ty1 ty2 e =
     | Tfloat Real, Tfloat Float -> make_app "r_to_s" [rounding_mode (); e]
     | Tfloat Real, Tfloat Double -> make_app "r_to_d" [rounding_mode (); e]
     | Tfloat Real, Tfloat LongDouble -> make_app "r_to_q" [rounding_mode (); e]
+    | Tfloat Float, Tfloat Real -> make_app "s_to_r" [e]
+    | Tfloat Double, Tfloat Real -> make_app "d_to_r" [e]
+    | Tfloat LongDouble, Tfloat Real -> make_app "q_to_r" [e]
     | Tfloat Float, Tfloat Double -> 
 	make_app "double_of_single" [e]
     | Tfloat Double, Tfloat Float -> 
@@ -958,6 +1063,9 @@ let interp_bin_op = function
   | Blt | Bgt | Ble | Bge | Beq | Bneq | Band | Bor ->
       assert false
 
+let int_one = Cte(Prim_int "1")
+let int_minus_one = Cte(Prim_int "(-1)")
+
 let any_float fk = 
   if floats then match fk with
     | Float -> "any_single"
@@ -967,9 +1075,6 @@ let any_float fk =
   else 
     "any_real"
 
-let int_one = Cte(Prim_int "1")
-let int_minus_one = Cte(Prim_int "(-1)")
-
 let float_of_real r fk = 
   if floats then match fk with
     | Float -> App (App (Var "r_to_s", rounding_mode ()), r)
@@ -1034,13 +1139,20 @@ let bin_op op t1 t2 = match op, t1, t2 with
   | _ ->
       build_minimal_app (interp_bin_op op) [t1; t2]
 
+let guarded_app f kind loc x y =
+  Output.Label (reg_loc ~kind loc, f x y)
 
+let guarded_make_app = guarded_app Output.make_app
+
+let guarded_build_complex_app = 
+  guarded_make_app (*guarded_app build_complex_app*)
 
 let rec interp_expr e =
   let w = interp_expr_loc e in
   if e.nexpr_loc = Loc.dummy_position then w else Loc (fst e.nexpr_loc, w)
 
 and interp_expr_loc e =
+  let loc = e.nexpr_loc in
   match e.nexpr_node with
     | NEconstant (IntConstant c) -> 
 	let t = Cte (Prim_int (Int64.to_string (Cconst.int e.nexpr_loc c))) in
@@ -1099,16 +1211,18 @@ and interp_expr_loc e =
 		else
 		  Let(tmp1, e1,
 		      Let(tmp2, interp_expr e2,
-			  append (build_complex_app (Var "upd_")
-				    [Var var; Var tmp1; Var tmp2])
+			  append (guarded_build_complex_app 
+				     PointerDeref loc "upd_"
+				     [Var var; Var tmp1; Var tmp2])
 			    (Var tmp2)))
 	    | HeapRef (Not_valid,var,e1) ->	
 		let tmp1 = tmp_var () in
 		let tmp2 = tmp_var () in
 		Let(tmp1, e1,
 		    Let(tmp2, interp_expr e2,
-			append (build_complex_app (Var "upd_")
-				  [Var var; Var tmp1; Var tmp2])
+			append (guarded_build_complex_app 
+				   PointerDeref loc "upd_"
+				   [Var var; Var tmp1; Var tmp2])
 			  (Var tmp2)))
 	end 
     | NEincr(op,e) -> 
@@ -1180,8 +1294,9 @@ and interp_expr_loc e =
 	    | Valid (a,b) ->
 		if (a<= Int64.zero && b>Int64.one)
 		then Output.make_app "safe_acc_" [Var(var);te] 
-		else Output.make_app "acc_" [Var(var);te]
-	    | Not_valid -> Output.make_app "acc_" [Var(var);te] 
+		else guarded_make_app PointerDeref loc "acc_" [Var(var);te]
+	    | Not_valid -> 
+		guarded_make_app PointerDeref loc "acc_" [Var(var);te] 
 	end
     | NEunary (Ustar, e) -> assert false
     | NEunary (Uplus, e) ->
@@ -1377,6 +1492,7 @@ and interp_address e = match e.nexpr_node with
       assert false (* not a left value *)
 
 and interp_statement_expr e =
+(*  let loc = e.nexpr_loc in*)
   match e.nexpr_node with
     | NEseq(e1,e2) ->
 	append (interp_statement_expr e1) (interp_statement_expr e2)
@@ -1388,16 +1504,14 @@ and interp_statement_expr e =
 	    | LocalRef(v) ->
 		Assign(v.var_unique_name,interp_expr e)
 	    | HeapRef(valid,var,e1) ->
-		begin
 		  match valid with 
-		    | Valid(a,b) -> if (a<= Int64.zero && Int64.zero <b) 
-		      then (build_complex_app (Var "safe_upd_")
-				  [Var var;e1; interp_expr e])
-		      else(build_complex_app (Var "upd_" )
-				      [Var var;e1; interp_expr e]) 
-		    | Not_valid -> (build_complex_app (Var "upd_" )
-				      [Var var;e1; interp_expr e])
-		end
+		    | Valid (a,b) when a<= Int64.zero && Int64.zero <b ->
+			build_complex_app
+			  (Var "safe_upd_") [Var var;e1; interp_expr e]
+		    | Valid _ | Not_valid -> 
+			guarded_build_complex_app PointerDeref l.nexpr_loc
+			  "upd_" 
+			  [Var var;e1; interp_expr e]
 	end 
     | NEincr(op,e) ->
 	let top,one = interp_incr_op e.nexpr_type op in
@@ -1618,41 +1732,43 @@ let rec make_union_loc = function
   | l::r -> LApp("pset_union",[l;make_union_loc r])
 
 let interp_assigns label old_label assigns = function
-  | Some locl ->
+  | Some (loc, locl) ->
       let m = HeapVarSet.fold
 	(fun v m -> 
-	   if Ceffect.is_alloc v then m 
-	   else StringMap.add (heap_var_name v) (Reference false) m)
+	  if Ceffect.is_alloc v then m 
+	  else StringMap.add (heap_var_name v) (Reference false) m)
 	assigns.Ceffect.assigns_var StringMap.empty 
       in
       let m = ZoneSet.fold
 	(fun (z,s,ty) m -> 
-	   StringMap.add (zoned_name s (Pointer z)) (Memory []) m)
+	  StringMap.add (zoned_name s (Pointer z)) (Memory []) m)
 	assigns.Ceffect.assigns m 
       in
-      
       let l = 
 	List.fold_left (collect_locations label old_label) m locl
       in
-      StringMap.fold
-	(fun v p acc -> match p with
-	   | Memory p ->
-	       if no_alloc_table then
-		 make_and acc
-		   (LPred("not_assigns",
+      let p = 
+	StringMap.fold
+	  (fun v p acc -> match p with
+	    | Memory p ->
+		if no_alloc_table then
+		  make_and acc
+		    (LPred("not_assigns",
 			  [interp_var (Some old_label) v;
 			   LVar v; make_union_loc p]))
-	       else
-		 make_and acc
-		   (LPred("not_assigns",
+		else
+		  make_and acc
+		    (LPred("not_assigns",
 			  [interp_var None "alloc"; 
 			   interp_var (Some old_label) v;
 			   LVar v; make_union_loc p]))
-	   | Reference false ->
-	       make_and acc (LPred("eq", [LVar v; interp_var (Some old_label) v]))
-	   | Reference true ->
-	       acc)
-	l LTrue
+	    | Reference false ->
+		make_and acc (LPred("eq", [LVar v; interp_var (Some old_label) v]))
+	    | Reference true ->
+		acc)
+	  l LTrue
+      in
+      named_predicate loc p
   | None ->
       LTrue
 
@@ -1812,7 +1928,7 @@ let interp_spec add_inv effect s =
    make_and
      (interp_predicate_opt None "" s.ensures)
      (make_and 
-	(interp_assigns (Some "") "" effect s.assigns)
+	(interp_assigns None "" effect s.assigns)
 	(make_and
 	   (if add_inv then weak_invariants_for effect else LTrue)
 	   (if Ceffect.assigns_alloc effect then alloc_extends () else LTrue)))
@@ -2093,7 +2209,10 @@ and interp_statement_loc ab may_break stat = match stat.nst_node with
       let post,_ = interp_predicate None "init" pred, Void in
       BlackBox (Annot_type (LTrue, unit_type, [], [], post, []))
   | NSlogic_label(l) -> 
-      Output.Label (l, Void)
+      assert false
+(*
+Output.Label (l, Void)
+*)
   | NSspec (spec,s) ->
       let eff = Ceffect.statement s in
       let pre,_,post = interp_spec false eff spec in
@@ -2130,12 +2249,16 @@ and interp_statement_loc ab may_break stat = match stat.nst_node with
       let decl = List.fold_left (fun acc x ->
 				   acc@[interp_statement ab may_break x]) 
 		   [] decl in
-      if v.var_is_assigned then
-	Let_ref(v.var_unique_name,tinit,
-		Block (decl@[interp_statement ab may_break rem]))
+      if v.var_is_static then (* if static then still globally declared *)
+	Block (decl@[interp_statement ab may_break rem])
       else
-	Let(v.var_unique_name,tinit,
-	    Block (decl@[interp_statement ab may_break rem]))
+	if v.var_is_assigned then
+	  Let_ref(v.var_unique_name,tinit,
+		 Block (decl@[interp_statement ab may_break rem]))
+	else
+	  Let(v.var_unique_name,tinit,
+	     Block (decl@[interp_statement ab may_break rem]))
+
 
 and interp_block ab may_break statements =
   let rec block = function
@@ -2151,6 +2274,9 @@ and interp_block ab may_break statements =
 	  let (be,bl) = block (st::bl) in
 	  Hashtbl.add labels_table lab.label_info_name ();
 	  Raise("Goto_"^lab.label_info_name,None),(lab,be)::bl
+    | {nst_loc = loc ; nst_node = NSlogic_label(l) } :: bl -> 
+	  let (be,bl) = block bl in
+	  Output.Label(l, be), bl
     | [s] ->
 	interp_statement ab may_break s,[]
     | { nst_node = NSnop } :: bl ->
@@ -2287,6 +2413,8 @@ let cinterp_logic_symbol id ls =
 	Predicate(false,id.logic_name, args,a)
     | NFunction(args,ret,_) ->
 	let ret_type =
+	  Info.output_why_type (Cenv.type_type_why ret false) 
+(*
 	  match ret.Ctypes.ctype_node with
 	    | Tvar s -> s
 	    | Tint _ -> "int"
@@ -2300,6 +2428,7 @@ let cinterp_logic_symbol id ls =
 		    | _ -> assert false
 		end
 	    | _ -> assert false
+*)
 	in
 	let args =
 	  List.fold_right
@@ -2320,10 +2449,12 @@ let cinterp_logic_symbol id ls =
             (fun (z,_,ty) t ->
                ("",Info.output_why_type (Info.Memory(ty,z)))::t)
             id.logic_heap_zone args in
-	Logic(false,id.logic_name,args,simple_logic_type ret_type)
+	Logic(false,id.logic_name,args,(*simple_logic_type*) ret_type)
     | NFunction_def(args,ret,e) ->
 	let e = interp_term None "" e in
 	let ret_type = 
+	  Info.output_why_type (Cenv.type_type_why ret false) 
+(*
 	  match ret.Ctypes.ctype_node with
 	    | Tvar s -> s
 	    | Tint _ -> "int"
@@ -2337,9 +2468,9 @@ let cinterp_logic_symbol id ls =
 		    | _ -> assert false
 		end
 	    | _ -> assert false
-	in
+*)	in
 	let args = interp_predicate_args id args in
-	Output.Function(false,id.logic_name,args,simple_logic_type ret_type,e)
+	Output.Function(false,id.logic_name,args,(* simple_logic_type *) ret_type,e)
 	  
 let interp_axiom p =
   let a = interp_predicate None "" p
