@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: clogic.mli,v 1.66 2007-05-18 09:29:12 filliatr Exp $ i*)
+(*i $Id: clogic.mli,v 1.69 2008/02/05 12:10:47 marche Exp $ i*)
 
 (* AST for C annotations *)
 
@@ -193,7 +198,7 @@ type 'term variant = 'term * string option
 
 type ('term,'pred) spec = { 
   mutable requires : 'pred option;
-  mutable assigns : 'term location list option;    
+  mutable assigns : (Loc.position * 'term location list) option;    
   mutable ensures : 'pred option;
   mutable decreases : 'term variant option
 }
@@ -202,7 +207,7 @@ type ('term,'pred) loop_annot = {
   invariant : 'pred option;
     (* part of the invariant already proved by program analysis *)
   assume_invariant : 'pred option;
-  loop_assigns : 'term location list option;
+  loop_assigns : (Loc.position * 'term location list) option;
   variant : 'term variant option
 }
 
