@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: cltyping.ml,v 1.118 2007-06-27 08:51:18 filliatr Exp $ i*)
+(*i $Id: cltyping.ml,v 1.125 2008/02/05 12:10:47 marche Exp $ i*)
 
 open Coptions
 open Format
@@ -60,7 +65,7 @@ let retype_typedef = function
   | Tint _ when not machine_ints -> Tint (Signed, ExactInt)
   | tn -> tn
 
-let rec type_logic_type loc env = function
+let rec type_logic_type ?(machine_ints=machine_ints) loc env = function
   | LTvoid -> c_void
   | LTchar _ | LTshort _ | LTint _ | LTlong _ | LTlonglong _ 
     when not machine_ints ->
@@ -75,8 +80,10 @@ let rec type_logic_type loc env = function
   | LTdouble -> use_floats := true; c_float Ctypes.Double
   | LTlongdouble -> use_floats := true; c_float Ctypes.LongDouble
   | LTreal -> c_real
-  | LTarray ty -> c_array Not_valid (type_logic_type loc env ty)
-  | LTpointer ty -> c_pointer Not_valid (type_logic_type loc env ty)
+  | LTarray ty -> 
+      c_array Not_valid (type_logic_type ~machine_ints loc env ty)
+  | LTpointer ty -> 
+      c_pointer Not_valid (type_logic_type ~machine_ints loc env ty)
   | LTvar id ->  
       noattr 
 	(try 
@@ -232,14 +239,14 @@ and type_term_node loc env = function
       let t = type_term env t in
       set_referenced t;
       Tunop (Uamp, t), noattr (Tpointer(Valid(Int64.zero,Int64.one), t.term_type))   
-  | PLunop (Unot, t) -> 
+  | PLnot t -> 
       let t = type_term env t in
       Tunop (Unot, t), t.term_type   
   | PLunop (Uround_error | Utotal_error | Uexact | Umodel as op, t) ->
       let t = type_float_term env t in
       Tunop (op, t), c_real
   | PLunop ((Ufloat_of_int | Uint_of_float | 
-	     Ufloat_conversion | Uint_conversion), _) ->
+	     Ufloat_conversion | Uint_conversion | Unot), _) ->
       assert false
   | PLbinop (t1, Badd, t2) ->
       let t1 = type_term env t1 in
@@ -251,9 +258,9 @@ and type_term_node loc env = function
 	    let t1,t2,ty = arith_conversion t1 t2 in
 	    Tbinop (t1, Badd, t2), ty
 	| (Tpointer _ | Tarray _), (Tint _ | Tenum _) -> 
-	    Tbinop (t1, Badd, t2), ty1
+	    Tbinop (t1, Badd, coerce c_exact_int t2), ty1
 	| (Tenum _ | Tint _), (Tpointer _ | Tarray _) ->
-	    Tbinop (t2, Badd, t1), ty2
+	    Tbinop (coerce c_exact_int t2, Badd, t1), ty2
 	| _ -> 
 	    error loc "invalid operands to binary +"
       end
@@ -333,8 +340,13 @@ and type_term_node loc env = function
 	 | _ ->
 	     error loc "subscripted value must be either array or pointer")
   | PLif (t1, t2, t3) ->
-      (* TODO type de t1 ? *)
-      unsupported loc "logic if-then-else"
+      let tt1 = type_term env t1 in
+      expected_num_pointer t1.lexpr_loc tt1;
+      let t2 = type_term env t2 in
+      let t3 = type_term env t3 in
+      expected_term_type loc t3 t2.term_type;
+      expected_term_type loc t2 t3.term_type;
+      Tif (tt1, t2, t3), t2.term_type
   | PLold t ->
       let t = type_term env t in
       Told t, t.term_type
@@ -376,7 +388,7 @@ and type_term_node loc env = function
       let t2 = type_int_term env t2 in
       Tmax (t1,t2), c_exact_int
   | PLminint ty ->
-      let ty = type_logic_type loc env ty in
+      let ty = type_logic_type ~machine_ints:true loc env ty in
       begin match ty.ctype_node with
 	| Tint (_, (Char | Short | Ctypes.Int | Long | LongLong)) -> 
 	    Tminint ty, c_exact_int
@@ -384,7 +396,7 @@ and type_term_node loc env = function
 	    error loc "argument must be a C integer type"
       end
   | PLmaxint ty ->
-      let ty = type_logic_type loc env ty in
+      let ty = type_logic_type ~machine_ints:true loc env ty in
       begin match ty.ctype_node with
 	| Tint (_, (Char | Short | Ctypes.Int | Long | LongLong)) -> 
 	    Tmaxint ty, c_exact_int
@@ -419,7 +431,7 @@ and type_term_node loc env = function
 	    warning loc "ignored cast in annotation"; t.term_node, tt
       end
   | PLvalid _ | PLvalid_index _ | PLvalid_range _ | PLfresh _ | PLseparated _ 
-  | PLexists _ | PLforall _ | PLnot _ | PLimplies _ | PLiff _ | PLfalse
+  | PLexists _ | PLforall _ | PLimplies _ | PLiff _ | PLfalse
   | PLfullseparated _ | PLor _ | PLand _ | PLrel _ | PLtrue | PLnamed _ 
   | PLbound_separated _ | PLfull_separated _ ->
       raise_located loc (AnyMessage "predicates are not allowed here")
@@ -763,7 +775,9 @@ let type_location = type_term
 let type_loop_annot env la =
   { invariant = option_app (type_predicate env) la.invariant;
     assume_invariant = option_app (type_predicate env) la.assume_invariant;
-    loop_assigns = option_app (List.map (type_location env)) la.loop_assigns;
+    loop_assigns = 
+      option_app (fun (loc,l) -> loc, List.map (type_location env) l) 
+	la.loop_assigns;
     variant = option_app (type_variant env) la.variant }
 
 let type_spec result env s = 
@@ -777,7 +791,7 @@ let type_spec result env s =
   in
   let q = option_app (type_predicate env') s.ensures in
   let v = option_app (type_variant env) s.decreases in
-  let m = option_app (List.map (type_location env)) s.assigns in
+  let m = option_app (fun (loc,l) -> loc, List.map (type_location env) l) s.assigns in
   { requires = p;
     assigns = m;
     ensures = q;
