@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: cmain.ml,v 1.97 2007-06-28 09:07:44 marche Exp $ i*)
+(*i $Id: cmain.ml,v 1.101 2008/02/05 12:10:47 marche Exp $ i*)
 
 open Format
 open Coptions
@@ -63,9 +68,6 @@ let main () =
   let nfiles = on_all_files Cnorm.file tfiles in
   (* local aliasing analysis *)
   if local_aliasing then Cptr.local_aliasing_transform ();
-  (* local abstract interpretation over integers *)
-  if abstract_interp || gen_invariant then
-    Array.iter Cint.local_int_analysis_attach tab_comp;
   (* separation *)  
   lprintf "starting separation of variables.@.";
   List.iter (fun (_,p) -> Cseparation.file p)  nfiles;
@@ -169,6 +171,8 @@ let main () =
       (fun fmt -> Output.fprintf_why_decls fmt why_code) (file ^ ".tmp");
     Lib.file_copy_if_different (file ^ ".tmp") (file ^ ".why")
   end;
+  (* print locs *)
+  Pp.print_in_file Cinterp.print_locs (Lib.file "." (first_file ^ ".loc"));
   (* makefile *)
   Cmake.makefile first_file;
   if show_time then
