@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: cprint.ml,v 1.47 2007-05-18 09:29:13 filliatr Exp $ i*)
+(*i $Id: cprint.ml,v 1.50 2008/02/05 12:10:47 marche Exp $ i*)
 
 (* Pretty-printer for normalized AST *)
 
@@ -409,7 +414,7 @@ let spec fmt = function
       ()
   | s ->
       let requires fmt p = fprintf fmt "@[requires %a@]@\n" npredicate p in
-      let assigns fmt a = fprintf fmt "@[assigns %a@]@\n" locations a in
+      let assigns fmt (_,a) = fprintf fmt "@[assigns %a@]@\n" locations a in
       let ensures fmt p = fprintf fmt "@[ensures %a@]@\n" npredicate p in
       let decreases fmt = function
 	| (t, None) -> fprintf fmt "@[decreases %a@]@\n" nterm t
@@ -429,7 +434,8 @@ let loop_annot fmt = function
       let invariant fmt p = fprintf fmt "@[invariant %a@]@\n" npredicate p in
       let assume_invariant fmt p = 
 	fprintf fmt "@[assume invariant %a@]@\n" npredicate p in
-      let loop_assigns fmt a = fprintf fmt "@[assigns %a@]@\n" locations a in
+      let loop_assigns fmt (_,a) = 
+	fprintf fmt "@[assigns %a@]@\n" locations a in
       let variant fmt = function
 	| (t, None) -> fprintf fmt "@[variant %a@]@\n" nterm t
 	| (t, Some r) -> fprintf fmt "@[variant %a for %s@]@\n" nterm t r
