@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: creport.ml,v 1.25 2007-04-26 13:41:19 filliatr Exp $ i*)
+(*i $Id: creport.ml,v 1.28 2008/02/05 12:10:47 marche Exp $ i*)
 
 open Format
 open Cerror
@@ -82,7 +87,7 @@ let report fmt = function
   | TooManyArguments ->
       fprintf fmt "Error: too many arguments"
   | PartialApp ->
-      fprintf fmt "Error: this application is partial"
+      fprintf fmt "Error: this application is partial (Argument missing?)"
   | Unsupported s ->
       fprintf fmt "Error: unsupported feature (%s)" s
 
