@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -393,22 +398,19 @@ let rec rehash z1 z2 =
     
 and unifier_type_why tw1 tw2 =
   match tw1,tw2 with
-    | Pointer z1 , Pointer z2 ->
-	unifier_zone z1 z2     
-    | Addr z1 , Addr z2 ->
-	unifier_zone z1 z2
-    | Why_Logic s1, Why_Logic s2 when s1 = s2 -> 
-	()
+    | Pointer z1 , Pointer z2 -> unifier_zone z1 z2     
+    | Addr z1 , Addr z2 -> unifier_zone z1 z2
+    | Why_Logic s1, Why_Logic s2 when s1 = s2 -> ()
     (* int types *)
     | Info.Int, Info.Int -> ()
-    | Why_Logic s1, Info.Int when is_int_type s1 -> ()
-    | Info.Int, Why_Logic s2 when is_int_type s2 -> ()
+    | Why_Logic s, Info.Int 
+    | Info.Int, Why_Logic s when is_int_type s -> ()
     | Why_Logic s1, Why_Logic s2 when is_int_type s1 && is_int_type s2 -> ()
     (* float types *)
-    | Why_Logic "double", Why_Logic "real"
-    | Why_Logic "single", Why_Logic "real" 
-    | Why_Logic "real", Why_Logic "single"
-    | Why_Logic "real", Why_Logic "double" -> ()
+    | Info.Real, Info.Real -> ()
+    | Why_Logic s, Info.Real 
+    | Info.Real, Why_Logic s when is_real_type s -> ()
+    | Why_Logic s1, Why_Logic s2 when is_real_type s1 && is_real_type s2 -> ()
     (* errors *)
     | Memory _, _ | _, Memory _ -> assert false
     | _ ->
@@ -417,7 +419,7 @@ and unifier_type_why tw1 tw2 =
 	in
 	Format.eprintf "anomaly: unify why types `%a' and `%a'@."
 	  Output.fprintf_logic_type t1 Output.fprintf_logic_type t2;
-	raise Not_found
+	assert false
 	  
 
 and unifier_zone z1 z2 =
@@ -437,10 +439,8 @@ and unifier_zone z1 z2 =
 	| _ -> assert false
     end
 
-let loc_name (b,e) =
-  let l = b.Lexing.pos_lnum in
-  let fc = b.Lexing.pos_cnum - b.Lexing.pos_bol + 1 in
-  let lc = e.Lexing.pos_cnum - b.Lexing.pos_bol + 1 in
+let loc_name loc =
+  let (f,l,fc,lc) = Loc.extract loc in
   Format.sprintf "line %d, characters %d-%d" l fc lc
 
 let unifier_type_why ?(var_name="?") tw1 tw2 =
@@ -492,6 +492,7 @@ let rec term tyf t =
 	       | Pointer z as ty -> 
 		   begin
 		     try
+		       let z = repr z in
 		       Pointer (assoc_zone z assoc)
 		     with
 			 Not_found -> ty
@@ -550,6 +551,7 @@ let rec predicate tyf p =
 		 | Pointer z as ty -> 
 		     begin
 		       try
+			 let z = repr z in
 			 Pointer (assoc_zone z assoc)
 		       with
 			   Not_found -> ty
@@ -690,7 +692,7 @@ let option_iter f x =
 
 let loop_annot tyf la =
   option_iter (predicate tyf) la.invariant;
-  option_iter (List.iter (term tyf)) la.loop_assigns;
+  option_iter (fun (_,l) -> List.iter (term tyf) l) la.loop_assigns;
   option_iter (fun (t,_) -> term tyf t) la.variant
 
 let spec tyf sp =
@@ -703,7 +705,7 @@ let spec tyf sp =
   begin
   match sp.assigns with
     | None -> ()
-    | Some l -> List.iter (term tyf) l
+    | Some (_,l) -> List.iter (term tyf) l
   end;
   begin
     match sp.ensures with
