@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: ctyping.ml,v 1.148 2007-04-26 13:41:19 filliatr Exp $ i*)
+(*i $Id: ctyping.ml,v 1.155 2008/02/05 12:10:47 marche Exp $ i*)
 
 open Format
 open Coptions
@@ -304,9 +309,21 @@ let float_constant_type s =
     | 'l' | 'L' -> String.sub s 0 (n-1), LongDouble
     | _ -> s, Double
 
+let has_suffix ~suf s =
+  let nsuf = String.length suf in
+  let n = String.length s  in
+  n >= nsuf && String.lowercase (String.sub s (n - nsuf) nsuf) = suf
+
 let int_constant_type s =
-   (* TODO *)
-   Signed, Int
+  if has_suffix ~suf:"ul" s || has_suffix ~suf:"lu" s then
+    Unsigned, Long
+  else if has_suffix ~suf:"u" s then
+    Unsigned, Int
+  else if has_suffix ~suf:"l" s then
+    Signed, Long
+  else
+    (* TODO: cf K&R *)
+    Signed, Int
 
 (* type the assignment of [e2] into a left value of type [ty1] *)
 let type_assignment loc ty1 e2 =
@@ -396,6 +413,10 @@ and type_type_node ?(ghost=false) ?(parameters=false) loc env ty =
       Tint (s, type_integer loc env i)
   | CTvar x -> 
       (* TODO: les attributs sont perdus *)
+      (* Cas d'erreur associs observs en pratique : 
+            -> utilisation d'un type non dclarer (pour dclarer une variable ghost 
+               ou la signature d'une fonction logique)
+            -> dclaration d'une variable ghost avec un type logique *)
       (try (find_typedef x).ctype_node with Not_found -> assert false)
   | CTarray (tyn, None) ->
       Tarray (Not_valid,type_type loc env tyn, None)
@@ -1207,7 +1228,9 @@ and type_statement_node loc env et lz = function
   | CSannot (Label l) ->
       TSlogic_label l, mt_status, lz
   | CSannot (GhostSet(x,t)) ->
-      TSset (type_ghost_lvalue env x,type_term env t), mt_status, lz
+      let x = type_ghost_lvalue env x in
+      let t = type_term env t in
+      TSset (x, Cltyping.coerce x.term_type t), mt_status, lz
   | CSspec (spec, s) ->
       let spec = type_spec env spec in
       let lz1,lz2 = match lz with
@@ -1292,7 +1315,7 @@ let type_spec_decl loc = function
       Taxiom (id, type_predicate (Env.empty ()) p)
   | LDinvariant (id, p) -> 
       Tinvariant (id, type_predicate (Env.empty ()) p)
-  | LDlogic (id, ty, pl, ll) ->
+  | LDlogic (id, ty, labels, pl, ll) ->
       let ty = type_logic_type loc (Env.empty ()) ty in
       let pl,env' = type_logic_parameters loc (Env.empty ()) pl in
       id.logic_args <- List.map fst pl;
@@ -1300,7 +1323,7 @@ let type_spec_decl loc = function
       let ll = List.map (type_location env') ll in
       Cenv.add_logic id.logic_name (List.map snd pl, ty, id);
       Tlogic (id, Function (pl, ty, ll))
-  | LDlogic_def (id, ty, pl, t) ->
+  | LDlogic_def (id, ty, labels, pl, t) ->
       let ty = type_logic_type loc (Env.empty ()) ty in
       let pl,env' = type_logic_parameters loc (Env.empty ()) pl in
       id.logic_args <- List.map fst pl;
