@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: info.ml,v 1.42 2007-04-27 13:24:26 filliatr Exp $ i*)
+(*i $Id: info.ml,v 1.48 2008/05/28 14:53:34 marche Exp $ i*)
 
 open Ctypes
 open Creport
@@ -32,6 +37,7 @@ type why_type =
   | Pointer of zone
   | Addr of zone
   | Int
+  | Real
   | Unit 
   | Why_Logic of string
 
@@ -41,7 +47,6 @@ and zone =
       number : int;
       mutable repr : zone option;
       name : string;
-(*      mutable type_why_zone : why_type;*)
     }
 
 
@@ -68,9 +73,11 @@ let rec same_why_type wt1 wt2 =
     | Memory(a1,z1), Memory(a2,z2) ->
 	same_zone z1 z2 && same_why_type a1 a2
     | Int, Int -> true
-(*    | Unit, Unit -> true *)
+    | Unit, Unit -> true 
+    | Real, Real -> true
     | Why_Logic s1, Why_Logic s2 -> s1=s2
-    | _, _ -> false
+    | Addr _, _ | _,Addr _ -> assert false
+    | _ -> false
 
 let rec same_why_type_no_zone wt1 wt2 =
   match wt1, wt2 with
@@ -78,9 +85,11 @@ let rec same_why_type_no_zone wt1 wt2 =
     | Memory (a1,_), Memory (a2,_) ->
 	same_why_type_no_zone a1 a2
     | Int, Int -> true
-(*     | Unit,Unit -> true *)
+    | Unit, Unit -> true 
+    | Real, Real -> true
     | Why_Logic s1, Why_Logic s2 -> s1=s2
-    | _, _ -> false
+    | Addr _, _ | _,Addr _ -> assert false
+    | _ -> false
 
 
 let found_repr ?(quote_var=true) z = 
@@ -101,6 +110,7 @@ let rec output_why_type ?(quote_var=true) ty=
   let rec output ty =
     match ty with
     | Int -> [], "int"
+    | Real -> [], "real"
     | Pointer z -> [output_zone_name ~quote_var z] , "pointer"    
     | Addr z -> [output_zone_name ~quote_var z] , "addr"
     | Memory(t,z) -> 
@@ -174,6 +184,21 @@ module HeapVarSet =
 		      i1.var_uniq_tag i2.var_uniq_tag 
 	   end)
 
+type label =
+  | Label_current
+  | Label_name of string
+
+module LabelSet = 
+  Set.Make(struct type t = label
+		  let compare = compare end)
+		    
+module HeapVarMap = 
+  Map.Make(struct type t = var_info 
+		  let compare i1 i2 = 
+		    Pervasives.compare
+		      i1.var_uniq_tag i2.var_uniq_tag 
+	   end)
+
 let print_hvs fmt s =
   HeapVarSet.iter (fun v -> Format.fprintf fmt "%s," v.var_unique_name) s
 
@@ -191,6 +216,14 @@ type logic_info =
       logic_name : string;
       mutable logic_heap_zone : ZoneSet.t;
       mutable logic_heap_args : HeapVarSet.t;
+(* 
+      mutable logic_heap_args : LabelSet.t HeapVarMap.t;
+
+   does not work because of hack in effect.mli, effect of logic funs:
+     reads_var = id.logic_heap_args;
+   which confuses global vars with heap vars 
+ 
+*)
       mutable logic_args : var_info list;
       mutable logic_why_type : why_type;
       mutable logic_args_zones : zone list;
