@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,13 +27,14 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: info.mli,v 1.37 2007-07-03 08:32:31 hubert Exp $ i*)
+(*i $Id: info.mli,v 1.41 2008-05-23 15:24:16 marche Exp $ i*)
 
 type why_type = 
   | Memory of why_type * zone
   | Pointer of zone
   | Addr of zone
   | Int
+  | Real
   | Unit 
   | Why_Logic of string 
 
@@ -91,15 +97,26 @@ val set_const_value : var_info -> int64 -> unit
 
 module HeapVarSet : Set.S with type elt = var_info
 
+module HeapVarMap : Map.S with type key = var_info
+
 module ZoneSet : Set.S with type elt = zone * string * why_type
 
 val print_hvs : Format.formatter -> HeapVarSet.t -> unit
 
+type label =
+  | Label_current
+  | Label_name of string
+
+module LabelSet : Set.S with type elt = label
+
 type logic_info =
     {
       logic_name : string;
       mutable logic_heap_zone : ZoneSet.t;
       mutable logic_heap_args : HeapVarSet.t;
+(* see the .ml
+      mutable logic_heap_args : LabelSet.t HeapVarMap.t;
+*)
       mutable logic_args : var_info list;
       mutable logic_why_type : why_type;
       mutable logic_args_zones : zone list;
