@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: invariant.ml,v 1.46 2007-04-27 13:24:26 filliatr Exp $ i*)
+(*i $Id: invariant.ml,v 1.49 2008/02/05 12:10:48 marche Exp $ i*)
 
 open Coptions
 open Creport
@@ -249,6 +254,24 @@ let rec fold_2 f l =
 let noattr_located n =  
   { Cast.node = n; Cast.loc = Loc.dummy_position }
 
+(*
+let heap_var_add v label m =
+  try
+    let l = HeapVarMap.find v m in
+    HeapVarMap.add v (LabelSet.add label l) m 
+  with Not_found ->
+    HeapVarMap.add v (LabelSet.singleton label) m 
+*)
+    
+let single v =  HeapVarSet.singleton v
+(*
+  heap_var_add v Label_current HeapVarMap.empty
+*)
+
+let pair v1 v2 = HeapVarSet.add v1 (single v2)
+(*
+  heap_var_add v1 Label_current (single v2)
+*)  
 
 let separation_first mark diag v1 v2 =
   let sep = if mark then "%separation1" else "%separation2" in
@@ -262,7 +285,7 @@ let separation_first mark diag v1 v2 =
     | Tstruct _ , Tstruct _ ->
 	let pre = sep ^ n1 ^ "_" ^ n2 in
 	let info = Info.default_logic_info (pre) in
-	info.logic_heap_args <- HeapVarSet.add v1 (HeapVarSet.singleton v2); 
+	info.logic_heap_args <- pair v1 v2;
 	Cenv.add_pred (pre)  ([], info);
 	[noattr_located (
 	   Cast.Ninvariant_strong (
@@ -272,7 +295,7 @@ let separation_first mark diag v1 v2 =
 	if compatible_type ty v2.var_type then
 	  let pre = sep ^ "_range1_" ^ n1 ^ "_" ^ n2 in 
 	  let info = Info.default_logic_info (pre) in
-	  info.logic_heap_args <- HeapVarSet.add v1 (HeapVarSet.singleton v2); 
+	  info.logic_heap_args <- pair v1 v2; 
 	  Cenv.add_pred (pre)  ([], info);
 	  [noattr_located 
 	     (Cast.Ninvariant_strong (
@@ -288,7 +311,7 @@ let separation_first mark diag v1 v2 =
 	if compatible_type ty v1.var_type then
 	  let pre = sep ^ "_range1_" ^ n1 ^ "_" ^ n2 in 
 	  let info = Info.default_logic_info (pre) in
-	  info.logic_heap_args <- HeapVarSet.add v1 (HeapVarSet.singleton v2); 
+	  info.logic_heap_args <- pair v1 v2; 
 	  Cenv.add_pred (pre)  ([], info);
 	  [noattr_located
 	     (Cast.Ninvariant_strong (
@@ -309,7 +332,7 @@ let separation_first mark diag v1 v2 =
 	if mark then
 	  let pre = sep ^ n1 ^ "_" ^ n2 in
 	  let info = Info.default_logic_info (pre) in
-	  info.logic_heap_args <- HeapVarSet.add v1 (HeapVarSet.singleton v2); 
+	  info.logic_heap_args <- pair v1 v2; 
 	  Cenv.add_pred (pre)  ([], info);
 	  let ty = noattr_type (Tpointer (Not_valid,noattr_type (Tvoid))) in
 	  let var = default_var_info (fresh_index()) in
@@ -330,7 +353,7 @@ let separation_first mark diag v1 v2 =
 	else
  	  let pre = sep ^ n1 ^ "_" ^ n2 in
 	  let info = Info.default_logic_info (pre) in
-	  info.logic_heap_args <- HeapVarSet.add v1 (HeapVarSet.singleton v2); 
+	  info.logic_heap_args <- pair v1 v2; 
 	  Cenv.add_pred (pre)  ([], info);
 	  let ty = noattr_type (Tpointer (Not_valid,noattr_type (Tvoid))) in
 	  let var1 = default_var_info (fresh_index()) in
@@ -398,7 +421,7 @@ let rec separation_intern n =
 	      | Tstruct _ -> 
 		  let pre = "%separation1_range_" ^ n1  in 
 		  let info = Info.default_logic_info (pre) in
-		  info.logic_heap_args <- HeapVarSet.singleton v1 ; 
+		  info.logic_heap_args <- single v1; 
 		  Cenv.add_pred (pre)  ([], info);
 		  [noattr_located (Cast.Ninvariant_strong (
 				    "internal_separation_" ^ n1 ^ "_array1" , 
@@ -413,7 +436,7 @@ let rec separation_intern n =
 	      | Tarray _ ->	
 		 let pre = "%separation1_range_" ^ n1  in 
 		 let info = Info.default_logic_info (pre) in
-		 info.logic_heap_args <- HeapVarSet.singleton v1 ; 
+		 info.logic_heap_args <- single v1 ; 
 		 Cenv.add_pred (pre)  ([], info);
 		 noattr_located (Cast.Ninvariant_strong (
 				   "internal_separation_" ^ n1 ^ "_array1" , 
@@ -461,7 +484,7 @@ let add_predicates l =
 	     | Tstruct n ->
 		 let pre = "%valid_acc_" ^ n  in 
 		 let info = Info.default_logic_info (pre) in
-		 info.logic_heap_args <- HeapVarSet.singleton f; 
+		 info.logic_heap_args <- single f; 
 		 Cenv.add_pred (pre)  ([], info);
 		 [noattr_located (
 		    Cast.Ninvariant_strong (
@@ -470,11 +493,11 @@ let add_predicates l =
 		 let n1 = f.var_unique_name in
 		 let pre = "%valid_acc_" ^ n1 in 
 		 let info = Info.default_logic_info (pre) in
-		 info.logic_heap_args <- HeapVarSet.singleton f; 
+		 info.logic_heap_args <- single f; 
 		 Cenv.add_pred (pre)  ([], info);
 		 let pre2 = "%valid_acc_range_" ^ n1  in 
 		 let info2 = Info.default_logic_info (pre2) in
-		 info2.logic_heap_args <- HeapVarSet.singleton f; 
+		 info2.logic_heap_args <- single f; 
 		 Cenv.add_pred (pre2)  ([], info2);
 		 noattr_located (
 		 Cast.Ninvariant_strong ("valid_array"^ n1,
