@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -82,6 +87,30 @@ let print_term fmt t =
 	fprintf fmt "(@[if %a then@ %a else@ %a@])" print0 a print0 b print0 c
     | Tapp (id, tl, _) -> 
 	fprintf fmt "@[%s(%a)@]" (Ident.string id) print_terms tl
+    | Tnamed (User n, t) ->
+	(match (Tools.grab_infos n) with
+	   | None -> 
+	       fprintf fmt "@[%a@]" print3 t
+	   | Some l ->
+	       let n = new_tag l in
+	       pp_open_tag fmt n;
+	       fprintf fmt "@[%a@]" print3 t;
+	       pp_close_tag fmt ()
+	)
+    | Tnamed (Internal n, t) ->
+	begin
+	  try
+	    let _xpl = Hashtbl.find Util.explanation_table n in
+	    (* TODO *)
+	    (* let n = new_tag xpl in
+	    pp_open_tag fmt n;
+	    *)
+	    fprintf fmt "@[%a@]" print3 t;
+	    (* pp_close_tag fmt () *)
+	    
+	  with
+	      Not_found -> fprintf fmt "@[%a@]" print3 t
+	end
   and print_terms fmt tl =
     print_list comma print3 fmt tl
   in
@@ -146,7 +175,7 @@ let print_predicate fmt p =
 	  print_pure_type t print0 p'
     | Pfpi _ ->
 	failwith "fpi not supported"
-    | Pnamed (n, p) ->
+    | Pnamed (User n, p) ->
 	(match (Tools.grab_infos n) with
 	   | None -> 
 	       fprintf fmt "@[%a@]" print3 p
@@ -156,6 +185,20 @@ let print_predicate fmt p =
 	       fprintf fmt "@[%a@]" print3 p;
 	       pp_close_tag fmt ()
 	)
+    | Pnamed (Internal n, p) ->
+	begin
+	  try
+	    let _xpl = Hashtbl.find Util.explanation_table n in
+	    (* TODO *)
+	    (* let n = new_tag xpl in
+	    pp_open_tag fmt n;
+	    *)
+	    fprintf fmt "@[%a@]" print3 p;
+	    (* pp_close_tag fmt () *)
+	    
+	  with
+	      Not_found -> fprintf fmt "@[%a@]" print3 p
+	end
     | (Por _ | Piff _ | Pand _ | Pif _ | Pimplies _ | Forallb _) as p -> 
 	fprintf fmt "@[(%a)@]" print0 p
   in
