@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -158,6 +163,13 @@ let ergo = {
   pr_id = Dispatcher.Ergo;
   pr_enc = NoEncoding;
   }
+let ergoSS = {
+  pr_name = "ergo(SS)";
+  pr_result = cols#add int;
+  pr_icon = cols#add GtkStock.conv;
+  pr_id = Dispatcher.Ergo;
+  pr_enc = SortedStratified;
+  }
 let cvc3 = {
   pr_name = "CVC3";
   pr_result = cols#add int;
@@ -174,7 +186,7 @@ let z3SS = {
   }
 
 let provers = [
-  simplify; ergo; z3SS ; yicesSStrat; cvc3; graph; 
+  ergo; (*ergoSS;*) simplify; z3SS ; yicesSStrat; cvc3; graph; 
   (*simplify_sstrat;*) simplify_strat; yices; rvsat; 
   zenon; (*zenon_pred; zenon_strat; zenon_rec;*)
   harvey; cvcl]
@@ -289,7 +301,7 @@ let _ =
 let create_model () =
   let model = GTree.tree_store cols in
   Dispatcher.iter
-    (fun ((_,s,_) as o) ->
+    (fun ((_,_,s,_) as o) ->
        Hashtbl.add obligs s o;
        let f,n = Tools.decomp_name s in
        let row =
