@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -113,8 +118,9 @@ let move_to_line line =
   if line <= !tv_source#buffer#line_count && line <> 0 then begin
     let it = !tv_source#buffer#get_iter (`LINE line) in 
     let mark = `MARK (!tv_source#buffer#create_mark it) 
-    and y = if !tv_source#buffer#line_count < 20 then 0.23 else 0.1 in
-    !tv_source#scroll_to_mark ~use_align:true ~yalign:y mark;
+    (* and y = if !tv_source#buffer#line_count < 20 then 0.23 else 0.1  *)
+    in
+    !tv_source#scroll_to_mark ~use_align:true ~yalign:0.5 mark;
     if debug then
       begin 
 	print_endline ("     [...] Moving to line n "^(string_of_int line)); 
@@ -122,8 +128,56 @@ let move_to_line line =
       end
   end
 
+let color_tag_table = Hashtbl.create 17
+
+let color_loc file (tv:GText.view) l b e =
+  let buf = tv#buffer in
+  let orange_bg = 
+    try
+      Hashtbl.find color_tag_table file
+    with Not_found ->
+      let t = buf#create_tag ~name:"orange_bg" [`BACKGROUND "orange"] in
+      Hashtbl.add color_tag_table file t;
+      t
+  in
+  buf#remove_tag orange_bg ~start:buf#start_iter ~stop:buf#end_iter;
+  let top = buf#start_iter in
+  let start = top#forward_lines (l-1) in
+  let start = start#forward_chars b in
+  let stop = start#forward_chars (e-b) in
+  buf#apply_tag ~start ~stop orange_bg
+  
+
+let banner () =
+"
+
+Welcome to GWhy (the Graphical VC viewer for the Why platform)
+This is Why version " ^ Version.version ^ 
+", compiled on " ^ Version.date ^ "
+Copyright (c) 2002-2007 ProVal team, INRIA
+This is free software with ABSOLUTELY NO WARRANTY (use option -warranty)"
+
 let move_to_source = function
-  | None -> ()
+  | None -> 
+      last_file := "";
+      begin
+	try
+	  !tv_source#set_buffer (Hashtbl.find files "")
+	with Not_found ->
+	  !tv_source#set_buffer (GText.buffer ());
+	  let b = !tv_source#buffer in
+	  b#set_text (banner());
+	  begin
+	    try
+	      let why_logo_image = Options.lib_file "why-logo-1.png" in
+	      let why_logo = GdkPixbuf.from_file why_logo_image in
+	      b#insert_pixbuf ~iter:b#start_iter ~pixbuf:why_logo 
+	    with _ -> 
+	      b#insert ~iter:b#start_iter "(Why logo: image not found)"	      
+	  end;
+	  b#insert ~iter:b#start_iter "\n\t";
+	  Hashtbl.add files "" b
+      end
   | Some loc ->
       let line = int_of_string loc.line
       and file = loc.file in
@@ -138,10 +192,17 @@ let move_to_source = function
 	  !tv_source#set_buffer (GText.buffer ());
 	  read_file (Some loc);
 	  Hashtbl.add files file !tv_source#buffer;
+	  Hashtbl.remove color_tag_table file;
 	end;
 	move_to_line line
       end
 
+ 
+let move_to_loc loc =
+  move_to_source (Some loc);
+  color_loc loc.file !tv_source (int_of_string loc.line) 
+    (int_of_string loc.sp) (int_of_string loc.ep)
+
 let rec intros ctx = function 
   | Forall (true, id, n, t, _, p) ->
       let id' = Ident.next_away id (predicate_vars p) in
@@ -239,7 +300,7 @@ let print_all (tbuf:GText.buffer) s p =
   (* 2. then we create the GTK tags and map them to the tag names *)
   create_all_tags tbuf;
   (* 3. then we fill the GTK text buffer using Tagsplit.split *)
-  let utags = split tbuf (Lexing.from_string str) in
+  let _utags = split tbuf (Lexing.from_string str) in
   let (_,concl) = p in
   let mytag = 
     tbuf#create_tag
@@ -259,7 +320,7 @@ let print_all (tbuf:GText.buffer) s p =
     create_all_tags tbuf;
     flush_str_formatter () 
   in
-  let utags' = split tbuf (Lexing.from_string conclusion) in
+  let _utags' = split tbuf (Lexing.from_string conclusion) in
   (*color_lines (utags @ utags');*)
   ()
 
@@ -309,11 +370,11 @@ let show_definition (tv:GText.view) (tv_s:GText.view) =
   let text = buf#get_text ~start ~stop () in
   if start <> stop && text <> "" then begin
     try 
-      let pos = fst (Loc.ident text) in
-      let loc = {file=pos.Lexing.pos_fname; 
-		 line=(string_of_int pos.Lexing.pos_lnum); 
-		 sp=(string_of_int pos.Lexing.pos_bol); 
-		 ep=(string_of_int pos.Lexing.pos_cnum)} 
+      let (f,l,b,e) = Loc.ident text in
+      let loc = {file=f; 
+		 line=(string_of_int l); 
+		 sp=(string_of_int b); 
+		 ep=(string_of_int e)} 
       in 
       (*
        * print_endline (text ^ "  " ^ (print_loc (Some(loc)))); 
@@ -323,7 +384,7 @@ let show_definition (tv:GText.view) (tv_s:GText.view) =
     with Not_found -> ()
   end
     
-let text_of_obligation (tv:GText.view) (o,s,p) = 
+let text_of_obligation (tv:GText.view) (o,expl,s,p) = 
   let p = p.Env.scheme_type in
   last_fct := s;
   if (unchanged s (is_active ())) then 
