@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: stat.ml,v 1.54 2007-07-04 15:21:30 filliatr Exp $ i*)
+(*i $Id: stat.ml,v 1.78 2008/04/15 09:00:07 filliatr Exp $ i*)
 
 open Printf
 open Options
@@ -34,8 +39,10 @@ open Pprinter
 let _ = gui := true
 let debug = ref Options.debug
 
+(*
 let is_caduceus = 
   List.exists (fun f -> Filename.basename f = "caduceus.why") Options.files 
+*)
 
 let get_files fl = 
   let files = 
@@ -179,28 +186,59 @@ let update_buffer tv =
   tv#set_buffer buf;
   buf
 
+open Logic_decl
+
+let show_xpl (loc,xpl) (tv:GText.view) =
+  match loc with
+    | (s,l,b,e) ->
+	let loc = 
+	  { Tags.file=s; 
+	    Tags.line= string_of_int l; 
+	    Tags.sp = string_of_int b; 
+	    Tags.ep = string_of_int e} in
+	Pprinter.move_to_loc loc;
+	let msg =
+	  match xpl.vc_kind with
+	    | EKLemma -> "Lemma"
+	    | k -> "VC: " ^ Explain.msg_of_kind k
+	in
+	!display_info ("file: " ^ (Filename.basename s) ^ " " ^ msg)
+
 let select_obligs (model:GTree.tree_store) (tv:GText.view) 
                   (tv_s:GText.view) selected_rows = 
   List.iter
     (fun p ->
        let row = model#get_iter p in
        let s = model#get ~row ~column:Model.fullname in
+       try
+	 let (loc,xpl,_,_) as o = Model.find_oblig s in
+	 let buf = update_buffer tv in
+	 buf#set_text "";
+	 Pprinter.text_of_obligation tv o;
+	 let mark = `MARK (tv#buffer#create_mark tv#buffer#end_iter) in
+	 tv#scroll_to_mark ~use_align:true mark;
+	 show_xpl (loc,xpl) tv_s  
+       with Not_found -> 
 	 try
-	   let o = Model.find_oblig s in
-	   let buf = update_buffer tv in
-	   buf#set_text "";
-	   Pprinter.text_of_obligation tv o;
-	   let mark = `MARK (tv#buffer#create_mark tv#buffer#end_iter) in
-	   tv#scroll_to_mark ~use_align:true mark
-	 with Not_found -> ())
+	   let (id,beh,(f,l,b,e)) = Hashtbl.find Util.program_locs s in
+	   let loc = 
+	     { Tags.file=f; 
+	       Tags.line= string_of_int l; 
+	       Tags.sp = string_of_int b; 
+	       Tags.ep = string_of_int e} in
+	   Pprinter.move_to_loc loc;
+	   !display_info ("file: " ^ (Filename.basename f) ^ " " ^ beh ^" of "^id);
+	 with Not_found ->
+	   Pprinter.move_to_source None;	 
+	   !display_info "(nothing selected ??)";
+	 
+    )
     selected_rows
 
 let prove fct = 
   ignore (Thread.create fct ())
 
-let update_statistics p (model:GTree.tree_store) row result = 
-  model#set ~row ~column:p.Model.pr_result result
-
+(*
 let get_statistics (model:GTree.tree_store) row = 
   let sl = 
     List.map 
@@ -208,6 +246,7 @@ let get_statistics (model:GTree.tree_store) row =
       (Model.get_provers ())
   in
   "[" ^ String.concat "|" sl ^ "]"
+*)
 
 let get_all_results f (model:GTree.tree_store) = 
   let mychildren = Model.find_fobligs f in
@@ -218,6 +257,22 @@ let get_all_results f (model:GTree.tree_store) =
     0
     mychildren
 
+let update_statistics (model:GTree.tree_store) row f children =
+(*
+  let statistics = get_statistics model row in
+*)
+  let total = string_of_int (get_all_results f model) in
+  let name =
+    if f="" then "User goals" else
+    try
+      let (id,beh,(f,l,b,e)) = Hashtbl.find Util.program_locs f in
+      beh ^ " of " ^ id
+    with Not_found -> "unknown function " ^ f
+  in
+  model#set 
+    ~row ~column:Model.name (name^" "^" ("^total^"/"^(string_of_int children)^")")
+
+
 let build_statistics (model:GTree.tree_store) f = 
   try
     let row = Model.find_fct f
@@ -235,11 +290,20 @@ let build_statistics (model:GTree.tree_store) f =
 	      and u = model#get ~row ~column:p.Model.pr_result in
 	      model#set ~row ~column:p.Model.pr_result (u+r)) 
 	   (Model.get_provers ()));
-    let statistics = get_statistics model row 
-    and total = string_of_int (get_all_results f model) 
+    update_statistics model row f !children
+      (*
+    let statistics = get_statistics model row in
+    let total = string_of_int (get_all_results f model) 
     and children = string_of_int !children in
+    let name =
+      try
+	let (id,beh,(f,l,b,e)) = Hashtbl.find Util.program_locs f in
+	beh ^ " of " ^ id
+      with Not_found -> "unknown function " ^ f
+    in
     model#set 
-      ~row ~column:Model.name (f^" "^" "^total^"/"^children^" "^statistics)
+      ~row ~column:Model.name (name^" "^" "^total^"/"^children)
+    *)
   with e -> 
     begin 
       print_endline ("     [...] Error: unexcepted exception: " ^ 
@@ -271,7 +335,7 @@ let try_proof oblig =
 let run_prover_child p (view:GTree.view) (model:GTree.tree_store) 
                      o bench alone = 
   let column_p = p.Model.pr_icon in
-  let (_, oblig, seq) = o in
+  let (_, _, oblig, seq) = o in
   if bench or (try_proof seq) then
     try 
       let row = 
@@ -283,6 +347,7 @@ let run_prover_child p (view:GTree.view) (model:GTree.tree_store)
 	  raise Exit
       in
       model#set ~row ~column:column_p `EXECUTE;
+      if !debug then Format.eprintf "oblig : %s@." oblig;
       let r = 
 	Dispatcher.call_prover 
 	  ~debug:!debug ~encoding:p.Model.pr_enc
@@ -386,7 +451,7 @@ let run_benchmark_fct (view:GTree.view) (model:GTree.tree_store) f () =
   try
     let row = Model.find_fct f in
     model#set ~row ~column:Model.total 0;
-    let n = model#iter_n_children (Some row) in
+    let _n = model#iter_n_children (Some row) in
     let mychildren = Model.find_fobligs f in
     Queue.iter
       (fun row -> 
@@ -411,7 +476,7 @@ let run_benchmark_fct (view:GTree.view) (model:GTree.tree_store) f () =
     end
 
 let run_benchmark (view:GTree.view) (model:GTree.tree_store) () =
-  Queue.iter (fun f -> prove (run_benchmark_fct view model f)) Model.fq
+  Queue.iter (fun f -> run_benchmark_fct view model f ()) Model.fq
 
 let main () = 
   let w = GWindow.window 
@@ -594,7 +659,7 @@ let main () =
   let _ = 
     proof_factory#add_image_item ~label:"Start _benchmark" 
       ~key:GdkKeysyms._B 
-      ~callback:(run_benchmark view model)
+      ~callback:(fun () -> prove (run_benchmark view model))
                 (***
                 (fun () -> 
 		   List.iter
@@ -850,10 +915,8 @@ let main () =
   let _ =
     files_combo#entry#event#connect#after#focus_in ~callback:
       begin fun ev -> 
-	let s = files_combo#entry#text in
-	let loc = {Tags.file=s; Tags.line="1"; Tags.sp="1"; Tags.ep="1"} in
-	Pprinter.move_to_source (Some loc);
-	Pprinter.reset_last_file ();
+	Pprinter.move_to_source None;
+	Pprinter.reset_last_file (); (* ?? pourquoi ?? *)
 	false
       end
   in
@@ -862,23 +925,23 @@ let main () =
    * Running obligation 
    *)
   ignore 
-    (view#event#connect#button_release ~callback:
-       (fun ev -> 
-	  if GdkEvent.Button.button ev = 3 then 
-	    List.iter 
-	      (fun p -> 
-		 let row = model#get_iter p in
-		 let s = model#get ~row ~column:Model.fullname in
-		 (if model#iter_has_child row then
-		    prove (run_prover_fct (Model.get_default_prover ()) 
-			     view model s false)
-		  else 
-		    prove (run_prover_oblig (Model.get_default_prover ()) 
-			     view model s false true));
-	      )
-	      view#selection#get_selected_rows;
-	  false
-       ));
+    (view#connect#row_activated ~callback:
+       (fun p col -> 
+	  let row = model#get_iter p in
+          let name = col#title in
+          let prover = 
+            try Model.get_prover name 
+            with Model.No_such_prover -> Model.get_default_prover ()
+          in
+	  let s = model#get ~row ~column:Model.fullname in
+	  (if model#iter_has_child row then
+	     prove (run_prover_fct prover 
+		      view model s false)
+	   else 
+	     prove (run_prover_oblig prover
+		      view model s false true));
+       )
+    );
 
   (*
    * Remove default pango menu for textviews 
@@ -928,12 +991,12 @@ let main () =
 	  select_obligs model tv1 tv2 [path]; 
   in
 
-  (* Setting special icons for prooved obligation in cache *)
+  (* Setting special icons for proved obligation in cache *)
   let _ = 
     load_cache "gwhy.cache";
     if not (Cache.is_empty ()) then 
       Hashtbl.iter 
-	(fun s (_,o,seq) -> 
+	(fun s (_,_,o,seq) -> 
 	   let cleaned = Cache.clean seq in
 	   if in_cache cleaned then begin 
 	     let row = Hashtbl.find Model.orows o in
@@ -966,11 +1029,7 @@ let main () =
   let _ = (* update statistics for functions *)
     Hashtbl.iter 
       (fun k row -> 
-	 let success = (string_of_int (get_all_results k model))
-	 and n = (string_of_int (model#iter_n_children (Some(row))))
-	 in let s = get_statistics model (Hashtbl.find Model.frows k)
-	 in model#set ~row ~column:Model.name (k^" "^" "^success^"/"^n^" "^s)
-      )
+	 update_statistics model row k (model#iter_n_children (Some row)))
       Model.frows
   in
   w#add_accel_group accel_group;
@@ -979,6 +1038,15 @@ let main () =
 
 (* Main *)
 let _ = 
+(*
   if not is_caduceus then Pprinter.desactivate ();
+*)
   main ();
   GtkThread.main ()
+
+	   
+(*
+Local Variables: 
+compile-command: "unset LANG; make -C .. bin/gwhy.byte"
+End: 
+*)
