@@ -1,8 +1,37 @@
+(**************************************************************************)
+(*                                                                        *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
+(*    Jean-Franois COUCHOT                                               *)
+(*    Mehdi DOGGUY                                                        *)
+(*    Jean-Christophe FILLITRE                                           *)
+(*    Thierry HUBERT                                                      *)
+(*    Claude MARCH                                                       *)
+(*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
+(*                                                                        *)
+(*  This software is free software; you can redistribute it and/or        *)
+(*  modify it under the terms of the GNU General Public                   *)
+(*  License version 2, as published by the Free Software Foundation.      *)
+(*                                                                        *)
+(*  This software is distributed in the hope that it will be useful,      *)
+(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
+(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
+(*                                                                        *)
+(*  See the GNU General Public License version 2 for more details         *)
+(*  (enclosed in the file GPL).                                           *)
+(*                                                                        *)
+(**************************************************************************)
+
 (***************************************************************************
 
 Abstract syntax trees for Java source files
 
-$Id: java_ast.mli,v 1.11 2007-06-19 15:44:10 marche Exp $
+$Id: java_ast.mli,v 1.36 2008-05-23 13:51:39 marche Exp $
 
 ***************************************************************************)
 
@@ -17,9 +46,11 @@ type qualified_ident = identifier list
 (*s Modifiers *)
  
 type modifier =
-    [ `STATIC | `FINAL | `GHOST | `PUBLIC | `PRIVATE | `PROTECTED | `NATIVE 
-    | `SYNCHRONIZED | `ABSTRACT | `TRANSIENT (* "threadsafe" ? *)
-    | `PURE | `SPEC_PUBLIC | `MODEL ]  
+  | Static | Final  | Public | Private | Protected | Native 
+  | Synchronized | Abstract | Transient (* "threadsafe" ? *) | Volatile
+  | Strictfp 
+  | Ghost | Model
+  | Non_null | Nullable | Annot_modifier of Lexing.position * string
 
 type modifiers = modifier list
 
@@ -35,9 +66,9 @@ type type_expr =
 type quantifier = Forall | Exists
 
 type variable_id =
-    | Simple_id of identifier
-    | Array_id of variable_id
-
+  | Simple_id of identifier
+  | Array_id of variable_id
+      
 
 type incr_decr_op = Preincr | Predecr | Postincr | Postdecr 
 
@@ -47,6 +78,7 @@ type bin_op =
     | Bbwand | Bbwor | Bbwxor
     | Blsl | Blsr | Basr
     | Beq | Bne | Bgt | Blt | Ble | Bge
+    | Bconcat (* + for Strings *)
 
 type un_op = Uplus | Uminus | Unot | Ucompl
 
@@ -75,26 +107,27 @@ and pexpr_node =
   | JPEif of pexpr * pexpr * pexpr
   | JPEthis
   | JPEfield_access of java_field_access
-  | JPEcall of pexpr option * identifier * pexpr list
+  | JPEcall_name of qualified_ident * pexpr list
+  | JPEcall_expr of pexpr * identifier * pexpr list
   | JPEsuper_call of identifier * pexpr list
   | JPEnew of qualified_ident * pexpr list
-  | JPEnew_array of array_creation
+  | JPEnew_array of type_expr * pexpr list 
+      (*r type, explicit dimensions *)
   | JPEarray_access of pexpr * pexpr
+  | JPEarray_range of pexpr * pexpr option * pexpr option
   | JPEcast of type_expr * pexpr
   | JPEinstanceof of pexpr * type_expr
       (* in annotations only *)
   | JPEresult  
   | JPEold of pexpr 
+  | JPEat of pexpr * identifier 
 (*
   | JPEfresh of expr 
   | Type of type_expr 
   | Typeof of expr 
 *)
-  | JPEquantifier of quantifier * type_expr * variable_id list * pexpr   
+  | JPEquantifier of quantifier * (type_expr * variable_id list) list * pexpr   
 
-and array_creation =
-  | Explicit_array_creation of pexpr * array_creation
-  | Implicit_array_creation of type_expr
 
 (*
 and set_ref = 
@@ -133,7 +166,7 @@ type 'a switch_label =
   | Default
 
 type parameter =
-  | Simple_parameter of type_expr * identifier
+  | Simple_parameter of modifier option * type_expr * identifier
   | Array_parameter of parameter
 
 type pstatement =
@@ -162,7 +195,7 @@ and pstatement_node =
   | JPSghost_local_decls of variable_declaration
   | JPSghost_statement of pexpr
   | JPSannot of Lexing.position * string
-  | JPSloop_annot of pexpr * pexpr
+  | JPSloop_annot of pexpr * pexpr option
 
 and block = pstatement list
 ;;
@@ -204,9 +237,9 @@ type constructor_declaration =
 
 type pbehavior =
     { java_pbehavior_assumes : pexpr option;
-      java_pbehavior_assigns : pexpr list option;
+      java_pbehavior_assigns : (Loc.position * pexpr list) option;
       java_pbehavior_throws : (qualified_ident * identifier option) option;
-      java_pbehavior_ensures : pexpr
+      java_pbehavior_ensures : pexpr 
     }
 
 type field_declaration =
@@ -217,11 +250,13 @@ type field_declaration =
   | JPFstatic_initializer of block
   | JPFannot of Lexing.position * string
   | JPFinvariant of identifier * pexpr
-  | JPFmethod_spec of pexpr option * (identifier * pbehavior) list
-;;
+  | JPFstatic_invariant of identifier * pexpr
+  | JPFmethod_spec of 
+      pexpr option * pexpr option * (identifier * pbehavior) list
+  | JPFclass of class_declaration
+  | JPFinterface of interface_declaration
 
-
-type class_declaration =
+and class_declaration =
     { 
       class_modifiers : modifiers;
       class_name : identifier;
@@ -232,20 +267,13 @@ type class_declaration =
 
 (*s interface declarations *)
 
-type interface_member_declaration =
-  | JPIMconstant of variable_declaration
-  | JPIMmethod of method_declaration
-  | Model_variable of variable_declaration
-  | JPIMannot of Lexing.position * string
-;;
-
-type interface_declaration =
+and interface_declaration =
     { interface_modifiers : modifiers;
       interface_name : identifier;
       interface_extends : qualified_ident list;
-      interface_members : interface_member_declaration list
+      interface_members : field_declaration list
     }
-;;
+
 
 (*s compilation units *)
 
@@ -253,10 +281,10 @@ type type_declaration =
   | JPTclass of class_declaration
   | JPTinterface of interface_declaration
   | JPTannot of Lexing.position * string
-  | JPTaxiom of identifier * pexpr
+  | JPTlemma of identifier * bool * logic_label list * pexpr
   | JPTlogic_type_decl of identifier 
-  | JPTlogic_reads of identifier * type_expr option * parameter list * pexpr list 
-  | JPTlogic_def of identifier * type_expr option * parameter list * pexpr 
+  | JPTlogic_reads of identifier * type_expr option * logic_label list * parameter list * pexpr list 
+  | JPTlogic_def of identifier * type_expr option * logic_label list * parameter list * pexpr 
 
 type import_statement =
   | Import_package of qualified_ident
