@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: java_options.ml,v 1.2 2007-06-18 07:15:58 marche Exp $ i*)
+(*i $Id: java_options.ml,v 1.18 2008/04/09 06:17:12 nrousset Exp $ i*)
 
 open Format
 
@@ -53,10 +58,24 @@ let libdir =
     lprintf "KRAKATOALIB is not set, using %s as default@." p;
     p
 
+let rec split ch s =
+  try
+    let i = String.index s ch in
+    let h = String.sub s 0 i
+    and t = String.sub s (i+1) (String.length s - i - 1)
+    in
+    h::(split ch t)
+  with
+    Not_found -> [s]
+     
+
 let libfile = "krakatoa.why"
 
-(*s command-line options *)
+let javacard = ref false
 
+      
+(*s command-line options *)
+      
 let parse_only = ref false
 let type_only = ref false
 let print_graph = ref false
@@ -65,6 +84,12 @@ let verbose = ref false
 let werror = ref false
 let why_opt = ref ""
 let ignore_overflow = ref false
+let nonnull_sem = ref Java_env.NonNullNone
+
+(* Jessie options *)
+let inv_sem = ref Jc_env.InvArguments
+let annotation_sem = ref Jc_env.AnnotNone
+let ai_domain = ref Jc_env.AbsNone
 
 let add_why_opt s = why_opt := !why_opt ^ " " ^ s
 
@@ -74,7 +99,7 @@ let files () = List.rev !files_
 
 let version () = 
   Printf.printf "This is Krakatoa version %s, compiled on %s
-Copyright (c) 2006 - ProVal INRIA project 
+Copyright (c) 2006-2007 - ProVal INRIA project 
 This is free software with ABSOLUTELY NO WARRANTY (use option -warranty)
 " Java_version.version Java_version.date;
   exit 0
@@ -101,8 +126,15 @@ let _ =
           "  treats warnings as errors";
 	"-version", Arg.Unit version,
           "  prints version and exit";
-	"-ignore-overflow", Arg.Set ignore_overflow,
-	  "  ignore arithmetic overflow threats" ;
+	"-javacard", Arg.Set javacard,
+	  "  source is Java Card";
+	"-nonnull-sem", Arg.String
+	  (function 
+	     | "none" -> nonnull_sem := Java_env.NonNullNone
+	     | "fields" -> nonnull_sem := Java_env.NonNullFields
+	     | "all" -> nonnull_sem := Java_env.NonNullAll
+	     | s -> raise (Arg.Bad ("Unknown nonnull_sem: " ^ s))),
+	"  <kind> nonnull-by-default semantics: none (default), fields, all"
       ]
       add_file usage
 
@@ -117,7 +149,22 @@ let debug = !debug
 let verbose = !verbose
 let werror = !werror
 let why_opt = !why_opt
-let ignore_overflow = !ignore_overflow
+
+let classpath = 
+  let p =
+    try
+      let v = Sys.getenv "KRAKATOACLASSPATH" in
+	lprintf "KRAKATOACLASSPATH is set to %s@." v;
+	split ':' v
+    with Not_found ->      
+      let p = Filename.concat libdir 
+	(if !javacard then "javacard_api" else "java_api") 
+      in
+	lprintf "KRAKATOACLASSPATH is not set, using %s as default@." p;
+	[p]
+  in
+    "." :: p 
+
 
 (*s error handling *)
 
