@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -25,43 +30,41 @@
 open Jc_env
 open Jc_fenv
 open Jc_ast
+open Jc_pervasives
+open Jc_iterators
 
 let rec term acc t =
-  match t.jc_term_node with
-    | JCTconst _ 
-    | JCTvar _ -> acc
-    | JCTapp (f,lt) -> f::(List.fold_left term acc lt)
-    | JCToffset(_,t,_) | JCTold t | JCTderef (t,_) -> term acc t
-    | JCTshift (t1,t2) 
-    | JCTrange (t1,t2)
-    | JCTbinary (t1,_,t2) -> term (term acc t1) t2
-    | JCTif(t1,t2,t3) -> term (term (term acc t1) t2) t3
-    | JCTcast(t,_)
-    | JCTinstanceof(t,_)
-    | JCTunary (_,t) -> term acc t
+  fold_term 
+    (fun acc t -> match t#node with
+    | JCTapp app -> app.jc_app_fun::acc
+    | _ -> acc
+    ) acc t
 
-let tag acc t = match t.jc_tag_node with
+let tag acc t = match t#node with
   | JCTtag _ | JCTbottom -> acc
   | JCTtypeof(t, _) -> term acc t
 
 let rec assertion acc p =
-  match p.jc_assertion_node with
+  match p#node with
   | JCAtrue 
   | JCAfalse -> acc
   | JCArelation(t1,op,t2) ->
       term (term acc t1) t2
-  | JCAapp(f,lt) -> f::(List.fold_left term acc lt)
+  | JCAapp app -> app.jc_app_fun :: (List.fold_left term acc app.jc_app_args)
   | JCAand(pl) | JCAor(pl) -> List.fold_left assertion acc pl
   | JCAimplies (p1,p2) | JCAiff(p1,p2) -> 
       assertion (assertion acc p1) p2
   | JCAif(t1,p2,p3) -> 
       assertion (assertion (term acc t1) p2) p3
-  | JCAnot p | JCAold p | JCAquantifier (_,_,p) -> assertion acc p
-  | JCAinstanceof(t,_)
+  | JCAnot p | JCAold p | JCAat(p,_) | JCAquantifier (_,_,p) -> 
+      assertion acc p
+  | JCAinstanceof(t,_,_)
   | JCAmutable(t,_,_)
   | JCAbool_term t -> term acc t
   | JCAtagequality(t1, t2, _) ->
       tag (tag acc t1) t2
+  | JCAmatch(t, pal) ->
+      term (List.fold_left (fun acc (_, a) -> assertion acc a) acc pal) t
 
 (*
 let spec s = 
@@ -89,30 +92,25 @@ let spec s =
 *)
 
 let loop_annot acc la = 
-  term (assertion acc la.jc_loop_invariant) la.jc_loop_variant
+  let acc = assertion acc la.jc_loop_invariant in
+  match la.jc_loop_variant with
+  | None -> acc
+  | Some t -> term acc t
 
-let rec statement acc s = 
-  match s.jc_statement_node with  
-    | JCScall (_, f, le, s) ->
-	let (a,b)=acc in statement (a,f::b) s
-    | JCSassign_var _ | JCSassign_heap _ -> acc
-    | JCSreturn _ | JCSpack _ | JCSunpack _ | JCSthrow _ -> acc
-    | JCSif(_, s1, s2) ->
-	statement (statement acc s1) s2
-    | JCSloop(spec,s) ->
-	let (a,b) = statement acc s in (loop_annot a spec,b)
-    | JCSblock sl -> 
-	List.fold_left statement acc sl
-    | JCStry (s, catches, finally) -> 
-	let acc =
-	  List.fold_left 
-	    (fun acc (_,_,s) -> statement acc s) 
-	    (statement acc s) catches
-	in statement acc finally
-    | JCSdecl(vi,_,s) -> 
-	statement acc s
-    | JCSassert _ -> acc
-    | JCSreturn_void -> acc
+let expr = 
+  IExpr.fold_left 
+    (fun acc e -> match e#node with  
+       | JCEapp call ->
+	   let f = call.jc_call_fun in
+	   let (a,b)=acc in
+	   begin match f with
+	     | JCfun f -> (a,f::b)
+	     | JClogic_fun f -> (f::a,b)
+	   end
+       | JCEloop(spec,s) ->
+	   let (a,b) = acc in (loop_annot a spec,b)
+       | _ -> acc
+    )
 
 let compute_logic_calls f t = 
   let calls =
@@ -124,9 +122,8 @@ let compute_logic_calls f t =
   f.jc_logic_info_calls <- calls
 
 let compute_calls f s b = 
-  let (a,b) = List.fold_left statement ([],[]) b in
+  let (a,b) = expr ([],[]) b in
   f.jc_fun_info_calls <- b
-
       
 module LogicCallGraph = struct 
   type t = (int, (logic_info * term_or_assertion)) Hashtbl.t 
@@ -145,7 +142,7 @@ module LogicCallGraph = struct
 module LogicCallComponents = Graph.Components.Make(LogicCallGraph)
 
 module CallGraph = struct 
-  type t = (int, (fun_info * fun_spec * statement list option)) Hashtbl.t
+  type t = (int, (fun_info * Loc.position * fun_spec * expr option)) Hashtbl.t
   module V = struct
     type t = fun_info
     let compare f1 f2 = Pervasives.compare f1.jc_fun_info_tag f2.jc_fun_info_tag
@@ -153,7 +150,7 @@ module CallGraph = struct
     let equal f1 f2 = f1 == f2
   end
   let iter_vertex iter =
-    Hashtbl.iter (fun _ (f,spec,b) -> iter f) 
+    Hashtbl.iter (fun _ (f,loc,spec,b) -> iter f) 
   let iter_succ iter _ f =
     List.iter iter f.jc_fun_info_calls 
   end
@@ -190,21 +187,21 @@ let compute_components table =
   Jc_options.lprintf "******************************\n";
   Array.iteri 
     (fun i l -> 
-       Jc_options.lprintf "Component %d:\n%a@." i
-	 (print_list newline 
-	    (fun fmt f -> fprintf fmt " %s calls: %a\n" f.jc_fun_info_name
-		 (print_list comma 
-		    (fun fmt f -> fprintf fmt "%s" f.jc_fun_info_name))
-		 f.jc_fun_info_calls))
-	 l)
+      Jc_options.lprintf "Component %d:\n%a@." i
+	(print_list newline 
+	   (fun fmt f -> fprintf fmt " %s calls: %a\n" f.jc_fun_info_name
+	       (print_list comma 
+		  (fun fmt f -> fprintf fmt "%s" f.jc_fun_info_name))
+	       f.jc_fun_info_calls))
+	l)
     tab_comp;
   tab_comp
-
-
+    
+    
 
 
 (*
 Local Variables: 
-compile-command: "make -C .. bin/jessie.byte"
+compile-command: "LC_ALL=C make -C .. bin/jessie.byte"
 End: 
 *)
