@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,18 +27,19 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(* $Id: jc_fenv.mli,v 1.13 2007-06-19 15:42:33 bardou Exp $ *)
+(* $Id: jc_fenv.mli,v 1.30 2008/04/10 16:05:55 moy Exp $ *)
 
 open Jc_env
 open Jc_envset
+open Jc_region
 
 (* When adding fields for effects, change accordingly function [same_effects]
    in [jc_effect.ml]. *)
 type effect =
     {
-      jc_effect_alloc_table : StringSet.t;
-      jc_effect_tag_table : StringSet.t;
-      jc_effect_memories : FieldSet.t;
+      jc_effect_alloc_table : StringRegionSet.t;
+      jc_effect_tag_table : VariantSet.t;
+      jc_effect_memories : LogicLabelSet.t FieldOrVariantRegionMap.t;
       jc_effect_globals : VarSet.t;
       jc_effect_mutable : StringSet.t;
       jc_effect_committed : StringSet.t;
@@ -52,28 +58,35 @@ type logic_info =
     {
       jc_logic_info_tag : int;
       jc_logic_info_name : string;
+      mutable jc_logic_info_final_name : string;
       mutable jc_logic_info_result_type : jc_type option;
          (*r None for predicates *)
+      mutable jc_logic_info_result_region : region;
       mutable jc_logic_info_parameters : var_info list;
+      mutable jc_logic_info_param_regions : region list;
       mutable jc_logic_info_effects : effect;
       mutable jc_logic_info_calls : logic_info list;
+      mutable jc_logic_info_labels : logic_label list;
     }
 
-type fun_info =
+type fun_info = 
     {
       jc_fun_info_tag : int;
       jc_fun_info_name : string;
-      jc_fun_info_return_type : jc_type;
+      jc_fun_info_final_name : string;
+      jc_fun_info_result : var_info;
+      jc_fun_info_return_region : region;
       mutable jc_fun_info_parameters : var_info list;
+      mutable jc_fun_info_param_regions : region list;
       mutable jc_fun_info_calls : fun_info list;
+      mutable jc_fun_info_is_recursive : bool;
       mutable jc_fun_info_logic_apps : logic_info list;
       mutable jc_fun_info_effects : fun_effect;
     }
-
-
-
+      
+      
 (*
-Local Variables: 
-compile-command: "make -C .. bin/jessie.byte"
-End: 
+  Local Variables: 
+  compile-command: "LC_ALL=C make -j -C .. bin/jessie.byte"
+  End: 
 *)
