@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,9 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(* $Id: jc_lexer.mli,v 1.5 2006-11-03 12:49:00 marche Exp $ *)
+(* $Id: jc_lexer.mli,v 1.8 2008/04/10 16:05:55 moy Exp $ *)
+
+open Jc_ast
 
 exception Lexical_error of Loc.position * string
 
@@ -30,6 +37,6 @@ exception Syntax_error of Loc.position
 
 val token : Lexing.lexbuf -> Jc_parser.token
 
-val parse  : string -> in_channel -> Jc_ast.pdecl list
+val parse  : string -> in_channel -> pexpr decl list
 
 
