@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,51 +27,16 @@
 (*                                                                        *)
 (**************************************************************************)
 
+(* $Id: jc_norm.mli,v 1.17 2008/04/10 16:05:55 moy Exp $ *)
+
 open Jc_env
 open Jc_fenv
 open Jc_ast
 
-(*
-val logic_type_table : (string,string) Hashtbl.t
-  
-val logic_functions_table : 
-  (int, logic_info * term_or_assertion) Hashtbl.t
-*)
-
-val functions_table : 
-  (int, fun_info * fun_spec * statement list option) Hashtbl.t
-
-val variables_table : 
-  (int, var_info * expr option) Hashtbl.t
-
-(*
-val structs_table : 
-  (string, struct_info * (logic_info * assertion) list) Hashtbl.t
-*)
-
-(*
-val enum_types_table : 
-  (string, enum_info) Hashtbl.t
-*)
-
-(*
-val axioms_table : 
-  (string, assertion) Hashtbl.t
-*)
-
-(*
-val exceptions_table : 
-  (string, exception_info) Hashtbl.t
-*)
-
-val code_function : fun_spec * tstatement list option 
-  -> fun_spec * statement list option
-
-val static_variable : var_info * texpr option -> var_info * expr option
-
+val decls : pexpr decl list -> nexpr decl list
 
 (*
 Local Variables: 
-compile-command: "make -C .. bin/jessie.byte"
+compile-command: "LC_ALL=C make -C .. bin/jessie.byte"
 End: 
 *)
