@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,9 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(* $Id *)
+(* $Id: jc_options.mli,v 1.27 2008/04/04 16:10:32 nrousset Exp $ *)
+
+
 
 (*s environment variables *)
 
@@ -39,11 +46,24 @@ val verbose : bool
 val werror : bool
 val why_opt : string
 
-val ai : bool
+val verify_all_offsets : bool
+val verify_invariants_only : bool
+val verify : string list
+
+val interprocedural : bool
+val main : string
 
 val files : unit -> string list 
 val usage : unit -> unit
 
+val inv_sem: Jc_env.inv_sem ref
+val separation_sem : Jc_env.separation_sem ref
+val annotation_sem : Jc_env.annotation_sem ref
+val ai_domain : Jc_env.abstract_domain ref
+val int_model : Jc_env.int_model ref
+val trust_ai : bool
+val fast_ai : bool
+
 (*s The log file *)
 
 val log : Format.formatter
@@ -56,4 +76,13 @@ exception Jc_error of Loc.position * string
 
 val parsing_error : Loc.position -> ('a, unit, string, 'b) format4 -> 'a
 
+val locs_table : 
+  (string, (string * int * int * int * Output.kind option * (string * Rc.rc_value) list)) 
+     Hashtbl.t
 
+
+(*
+Local Variables: 
+compile-command: "make -C .. bin/jessie.byte"
+End: 
+*)
