@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: output.ml,v 1.12 2007-07-06 07:45:28 marche Exp $ i*)
+(*i $Id: output.ml,v 1.32 2008/05/07 09:54:00 filliatr Exp $ i*)
 
 open Lexing
 open Format
@@ -33,21 +38,26 @@ type constant =
   | Prim_int of string
   | Prim_real of string
   | Prim_bool of bool
-;;
+(*
+  | Prim_string of string
+*)
 
 let fprintf_constant form e =
   match e with
     | Prim_void -> fprintf form "void"
-    | Prim_int(n) -> fprintf form "%s" n
+    | Prim_int(n) -> fprintf form "(%s)" n
     | Prim_real(f) -> fprintf form "%s" f
     | Prim_bool(b) -> fprintf form "%b" b
-;;
+(*
+    | Prim_string s -> fprintf form "\"%s\"" s
+*)
 
 type term = 
   | LConst of constant
   | LApp of string * term list
   | LVar of string
   | LVarAtLabel of string * string     (*r x@L *)
+  | Tnamed of string * term
 ;;
 
 let rec iter_term f t =
@@ -56,6 +66,7 @@ let rec iter_term f t =
   | LApp(id,l) -> f id; List.iter (iter_term f) l
   | LVar(id) -> f id
   | LVarAtLabel(id,l) -> f id
+  | Tnamed(_,t) -> iter_term f t
 ;;
 
 let rec fprintf_term form t =
@@ -76,7 +87,7 @@ let rec fprintf_term form t =
   | LApp(id,[])
   | LVar(id) -> fprintf form "%s" id
   | LVarAtLabel(id,l) -> fprintf form "%s@@%s" id l
-;;
+  | Tnamed(lab,t) -> fprintf form "(%s : %a)" lab fprintf_term t
 
 type logic_type = 
     { logic_type_name : string;
@@ -106,8 +117,13 @@ type assertion =
   | LNamed of string * assertion
 ;;
 
+let rec unname a =
+  match a with
+    | LNamed(_,a) -> unname a
+    | _ -> a
+
 let make_or a1 a2 =
-  match (a1,a2) with
+  match (unname a1,unname a2) with
     | (LTrue,_) -> LTrue
     | (_,LTrue) -> LTrue
     | (LFalse,_) -> a2
@@ -115,8 +131,8 @@ let make_or a1 a2 =
     | (_,_) -> LOr(a1,a2)
 
 let make_and a1 a2 =
-  match (a1,a2) with
-    | (LTrue,_) -> a2
+  match (unname a1,unname a2) with
+    | (LTrue, _) -> a2
     | (_,LTrue) -> a1
     | (LFalse,_) -> LFalse
     | (_,LFalse) -> LFalse
@@ -133,7 +149,7 @@ let rec make_or_list l =
     | f::r -> make_or f (make_or_list r)
 
 let make_impl a1 a2 =
-  match (a1,a2) with
+  match (unname a1,unname a2) with
     | (LTrue,_) -> a2
     | (_,LTrue) -> LTrue
     | (LFalse,_) -> LTrue
@@ -141,7 +157,7 @@ let make_impl a1 a2 =
     | (_,_) -> LImpl(a1,a2)
 
 let make_equiv a1 a2 =
-  match (a1,a2) with
+  match (unname a1,unname a2) with
     | (LTrue,_) -> a2
     | (_,LTrue) -> a1
     | (_,_) -> LIff(a1,a2)
@@ -220,8 +236,7 @@ let rec fprintf_assertion form a =
       fprintf form "@[%s(%a" id fprintf_term t;
       List.iter (fun t -> fprintf form ",@ %a" fprintf_term t) tl;
       fprintf form ")@]"
-  | LPred(id,l) -> 
-      assert false
+  | LPred (id, []) -> assert false
   | LNamed (n, a) ->
       fprintf form "@[(%s:@ %a)@]" n fprintf_assertion a
 ;;
@@ -492,7 +507,7 @@ let rec fprintf_expr form e =
     | Block(el) ->
 	fprintf form "@[<hv 0>begin@ @[<hv 1>  %a@]@ end@]" fprintf_expr_list el
     | Assign(id,e) ->
-	fprintf form "@[<hv 1>%s := %a@]" 
+	fprintf form "@[<hv 1>(%s := %a)@]" 
 	  id fprintf_expr e
     | Let(id,e1,e2) ->
 	fprintf form "@[<hv 0>(let %s = %a in@ %a)@]" id
@@ -565,7 +580,7 @@ let rec fprintf_expr form e =
 	fprintf form "@[<hv 0>(assert@ { %a };@ %a)@]" 
 	  fprintf_assertion p fprintf_expr e
     | Label (s, e) ->
-	fprintf form "@[<hv 0>%s:@ %a@]" s fprintf_expr e
+	fprintf form "@[<hv 0>(%s:@ %a)@]" s fprintf_expr e
     | BlackBox(t) ->
 	fprintf form "@[<hv 0>[ %a ]@]" 
 	  (fprintf_type false) t
@@ -598,6 +613,7 @@ type why_decl =
   | Def of string * expr               (*r global let in why *)
   | Logic of bool * string * (string * logic_type) list * logic_type    (*r logic decl in why *)
   | Axiom of string * assertion         (*r Axiom *)
+  | Goal of string * assertion         (*r Goal *)
   | Predicate of bool * string * (string * logic_type) list * assertion  
   | Function of bool * string * (string * logic_type) list * logic_type * term
   | Type of string * string list
@@ -607,13 +623,14 @@ type why_decl =
 
 let get_why_id d =
   match d with
-    | Param(_,id,_) -> id
-    | Logic(_,id,_,_) -> id
-    | Def(id,_) -> id
-    | Axiom(id,_) -> id
-    | Predicate(_,id,_,_) -> id
-    | Function(_,id,_,_,_) -> id
-    | Type (id,_) -> id
+    | Param(_,id,_) 
+    | Logic(_,id,_,_)
+    | Def(id,_) 
+    | Axiom(id,_) 
+    | Goal(id,_) 
+    | Predicate(_,id,_,_) 
+    | Function(_,id,_,_,_) 
+    | Type (id,_) 
     | Exception(id,_) -> id
 
 let iter_why_decl f d =
@@ -623,7 +640,7 @@ let iter_why_decl f d =
     | Logic(_,id,args,t) -> 
 	List.iter (fun (_,t) -> iter_logic_type f t) args;
 	iter_logic_type f t
-    | Axiom(id,t) -> iter_assertion f t
+    | Goal(id,t) | Axiom(id,t) -> iter_assertion f t
     | Predicate(_,id,args,p) -> 
 	List.iter (fun (_,t) -> iter_logic_type f t) args;
 	iter_assertion f p
@@ -650,7 +667,7 @@ let rec do_topo decl_map iter_fun output_fun id d =
   match d.state with
     | `DONE -> ()
     | `RUNNING -> 
-	eprintf "Recursive definition of %s in generated file" id
+	eprintf "Warning: recursive definition of %s in generated file@." id
     | `TODO ->
 	d.state <- `RUNNING;
 	iter_fun
@@ -692,22 +709,21 @@ let fprintf_why_decl form d =
     | Axiom(id,p) ->
 	fprintf form "@[<hv 1>axiom %s :@ %a@]@.@." id 
 	  fprintf_assertion p
+    | Goal(id,p) ->
+	fprintf form "@[<hv 1>goal %s :@ %a@]@.@." id 
+	  fprintf_assertion p
     | Def(id,e) ->
 	fprintf form "@[<hv 1>let %s =@ %a@]@.@." id fprintf_expr e
-    | Predicate(b,id,[],p) -> assert false
-    | Predicate(b,id,a::args,p) ->
-	fprintf form "@[<hv 1>%spredicate %s(%a"
+    | Predicate (b, id, args, p) ->
+	fprintf form "@[<hv 1>%spredicate %s(%a) =@ %a@]@.@."
 	  (if b then "external " else "") id 
-	  fprint_logic_arg a;
-	List.iter (fun a -> fprintf form ",%a" fprint_logic_arg a) args;
-	fprintf form ") =@ %a@]@.@." fprintf_assertion p
-    | Function(b,id,[],t,e) -> assert false 
-    | Function(b,id,a::args,t,e) ->
-	fprintf form "@[<hv 1>%sfunction %s(%a"
+	  (print_list comma fprint_logic_arg) args
+	  fprintf_assertion p
+    | Function(b,id,args,t,e) ->
+	fprintf form "@[<hv 1>%sfunction %s(%a) : %a =@ %a@]@.@."
 	  (if b then "external " else "") id 
-	  fprint_logic_arg a;
-	List.iter (fun a -> fprintf form ",%a" fprint_logic_arg a) args;
-	fprintf form ") : %a =@ %a@]@.@." fprintf_logic_type t 
+	  (print_list comma fprint_logic_arg) args
+	  fprintf_logic_type t 
 	  fprintf_term e
     | Type (id, []) ->
 	fprintf form "@[type %s@]@.@." id
@@ -735,31 +751,128 @@ let fprintf_why_decls form decls =
   (* Why do we need a partition ?
      because one may have a type and a logic/parameter with the same name, 
      and the computation of dependencies is confused in that case
-
+     
      Type may depend on nothing 
      Logic may depend on Type, Logic and Predicate
      Predicate may depend on Type, Predicate and Logic
      Axiom may depend on Type, Predicate and Logic
      Parameter may depend on Type, Predicate and Logic
      Def may depend on Type, Parameter, Predicate, Logic, and Def
-
+     
      - Claude, 16 nov 2006
+
   *)
-  let (types,defs,others) =
+
+(*
+  let (types, defs, others) =
     List.fold_left
-    (fun (t,d,o) decl ->
-       match decl with
-	 | Type _ -> (decl::t,d,o)
-	 | Def _ -> (t,decl::d,o)
-	 | _ -> (t,d,decl::o))
-    ([],[],[]) decls
+      (fun (t,d,o) decl ->
+	 match decl with
+	   | Type _ -> (decl::t,d,o)
+	   | Def _ -> (t,decl::d,o)
+	   | _ -> (t,d,decl::o))
+      ([],[],[]) decls
   in
-  output_decls get_why_id iter_why_decl (fprintf_why_decl form) types;
-  output_decls get_why_id iter_why_decl (fprintf_why_decl form) others;
-  output_decls get_why_id iter_why_decl (fprintf_why_decl form) defs
+    output_decls get_why_id iter_why_decl (fprintf_why_decl form) types;
+    output_decls get_why_id iter_why_decl (fprintf_why_decl form) others;
+    output_decls get_why_id iter_why_decl (fprintf_why_decl form) defs
+*)
+
+  (*
+    Additional rules :
+    
+    Exception may depend on Type
+    Parameter may depend on Exception
+    
+    - Nicolas R., 8 nov 2007
+    
+  *)
+
+  let (types, params, defs, others) =
+    List.fold_left
+      (fun (t, p, d, o) decl ->
+	 match decl with
+	   | Type _ -> (decl::t, p, d, o)
+	   | Param _ -> (t, decl::p, d, o)
+	   | Def _ -> (t, p, decl::d, o)
+	   | _ -> (t, p, d, decl::o))
+      ([], [], [], []) decls
+  in
+    output_decls get_why_id iter_why_decl (fprintf_why_decl form) types;
+    output_decls get_why_id iter_why_decl (fprintf_why_decl form) others;
+    output_decls get_why_id iter_why_decl (fprintf_why_decl form) params;
+    output_decls get_why_id iter_why_decl (fprintf_why_decl form) defs
 
 
+(*s locs table *)
+
+type kind =
+  | ArithOverflow
+  | DownCast
+  | IndexBounds
+  | PointerDeref
+  | UserCall
+  | DivByZero
+  | AllocSize
+  | Pack
+  | Unpack
 
 
+let locs_table : 
+    (string, (kind option * string option * string option * Loc.position)) 
+    Hashtbl.t 
+    = Hashtbl.create 97
+let name_counter = ref 0
+let reg_loc prefix ?id ?kind ?name ?formula loc =
+  let id = match id with
+    | None ->  
+	incr name_counter;
+	prefix ^ "_" ^ string_of_int !name_counter
+    | Some n -> n
+  in
+  Hashtbl.add locs_table id (kind,name,formula,loc);
+  id
 
+let print_kind fmt k =
+  fprintf fmt "%s"
+    (match k with
+       | Pack -> "Pack"
+       | Unpack -> "Unpack"
+       | DivByZero -> "DivByZero"
+       | AllocSize -> "AllocSize"
+       | UserCall -> "UserCall"
+       | PointerDeref -> "PointerDeref"
+       | IndexBounds -> "IndexBounds"
+       | DownCast -> "DownCast"
+       | ArithOverflow -> "ArithOverflow"
+    )
 
+let abs_fname f =
+  if Filename.is_relative f then
+    Filename.concat (Unix.getcwd ()) f 
+  else f
+
+let print_locs fmt =
+  Hashtbl.iter 
+    (fun id (kind,name,formula,(b,e)) ->
+       fprintf fmt "[%s]@\n" id;
+       Option_misc.iter
+	 (fun k -> fprintf fmt "kind = %a@\n" print_kind k) kind;
+       Option_misc.iter
+	 (fun n -> fprintf fmt "name = \"%s\"@\n" n) name;
+       Option_misc.iter
+	 (fun n -> fprintf fmt "formula = \"%s\"@\n" n) formula;
+       fprintf fmt "file = \"%s\"@\n" (String.escaped (abs_fname b.Lexing.pos_fname));
+       let l = b.Lexing.pos_lnum in
+       let fc = b.Lexing.pos_cnum - b.Lexing.pos_bol in
+       let lc = e.Lexing.pos_cnum - b.Lexing.pos_bol in
+       fprintf fmt "line = %d@\n" l;
+       fprintf fmt "begin = %d@\n" fc;
+       fprintf fmt "end = %d@\n@\n" lc)
+    locs_table
+
+(*
+  Local Variables: 
+  compile-command: "LC_ALL=C make -j -C .. bin/jessie.byte"
+  End: 
+*)
