@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,14 +27,16 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: output.mli,v 1.11 2007-06-14 10:43:14 bardou Exp $ i*)
+(*i $Id: output.mli,v 1.20 2008/04/02 08:38:13 marche Exp $ i*)
 
 type constant =
   | Prim_void
   | Prim_int of string
   | Prim_real of string
   | Prim_bool of bool
-;;
+(*
+  | Prim_string of string
+*)
 
 type logic_type = 
     { logic_type_name : string;
@@ -44,6 +51,7 @@ type term =
   | LApp of string * term list
   | LVar of string
   | LVarAtLabel of string * string     (*r x@L *)
+  | Tnamed of string * term
 ;;
 
 val fprintf_term : Format.formatter -> term -> unit
@@ -173,6 +181,7 @@ type why_decl =
   | Def of string * expr               (*r global let in why *)
   | Logic of bool * string * (string * logic_type) list * logic_type  (*r logic decl in why *)
   | Axiom of string * assertion            (*r Axiom *)
+  | Goal of string * assertion            (*r Goal *)
   | Predicate of bool * string * (string * logic_type) list * assertion  
   | Function of bool * string * (string * logic_type) list * logic_type * term
   | Type of string * string list
@@ -182,5 +191,23 @@ val fprintf_why_decl : Format.formatter -> why_decl -> unit;;
 
 val fprintf_why_decls : Format.formatter -> why_decl list -> unit
 
+type kind =
+  | ArithOverflow
+  | DownCast
+  | IndexBounds
+  | PointerDeref
+  | UserCall
+  | DivByZero
+  | AllocSize
+  | Pack
+  | Unpack
 
+val locs_table : 
+    (string, (kind option * string option * string option * Loc.position)) 
+    Hashtbl.t 
+
+val reg_loc : string -> ?id:string -> ?kind:kind -> ?name:string
+  -> ?formula:string -> Loc.position -> string
+
+val print_locs : Format.formatter -> unit
 
