@@ -1,3 +1,31 @@
+(**************************************************************************)
+(*                                                                        *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
+(*    Jean-Franois COUCHOT                                               *)
+(*    Mehdi DOGGUY                                                        *)
+(*    Jean-Christophe FILLITRE                                           *)
+(*    Thierry HUBERT                                                      *)
+(*    Claude MARCH                                                       *)
+(*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
+(*                                                                        *)
+(*  This software is free software; you can redistribute it and/or        *)
+(*  modify it under the terms of the GNU General Public                   *)
+(*  License version 2, as published by the Free Software Foundation.      *)
+(*                                                                        *)
+(*  This software is distributed in the hope that it will be useful,      *)
+(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
+(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
+(*                                                                        *)
+(*  See the GNU General Public License version 2 for more details         *)
+(*  (enclosed in the file GPL).                                           *)
+(*                                                                        *)
+(**************************************************************************)
 
 (* Mix to Why *)
 
@@ -63,6 +91,12 @@ let interp_instr loc i op = match i with
 	| Mul -> "mul_int" | Div -> "div_int" | _ -> assert false
       in
       Wassign ("a", Tapp (aop, [Tvar "!a"; value_at loc op]))
+  | Srb ->
+      begin match op.pop_address, op.pop_index, op.pop_field with
+	| Some (PAconst "1"), None, None -> 
+	    Wassign ("a", Tapp ("div_int", [Tvar "!a"; Tconst "2"]))
+	| _ -> error loc UnsupportedInstruction
+      end
   | Cmp r -> 
       Wassign ("cmp", 
 	       Tapp ("sub_int", [Tvar (register_value r); value_at loc op]))
@@ -72,10 +106,10 @@ let interp_instr loc i op = match i with
       Wassign (register_name r, Tapp ("neg_int", [value_op loc op]))
   | Inc r ->
       Wassign (register_name r, 
-	      Tapp ("add_int", [Tvar (register_value r); Tconst "1"]))
+	      Tapp ("add_int", [Tvar (register_value r); value_op loc op]))
   | Dec r ->
       Wassign (register_name r, 
-	       Tapp ("sub_int", [Tvar (register_value r); Tconst "1"]))
+	       Tapp ("sub_int", [Tvar (register_value r); value_op loc op]))
   | Nop ->
       Wvoid
   | Jmp | Jsj 
