@@ -1,3 +1,31 @@
+(**************************************************************************)
+(*                                                                        *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
+(*    Jean-Franois COUCHOT                                               *)
+(*    Mehdi DOGGUY                                                        *)
+(*    Jean-Christophe FILLITRE                                           *)
+(*    Thierry HUBERT                                                      *)
+(*    Claude MARCH                                                       *)
+(*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
+(*                                                                        *)
+(*  This software is free software; you can redistribute it and/or        *)
+(*  modify it under the terms of the GNU General Public                   *)
+(*  License version 2, as published by the Free Software Foundation.      *)
+(*                                                                        *)
+(*  This software is distributed in the hope that it will be useful,      *)
+(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
+(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
+(*                                                                        *)
+(*  See the GNU General Public License version 2 for more details         *)
+(*  (enclosed in the file GPL).                                           *)
+(*                                                                        *)
+(**************************************************************************)
 
 (* Turn a MIX program into a set of sequential MIX programs *)
 
@@ -159,8 +187,18 @@ let interp_stmt self prev lab s = match s.node with
 	| _ -> assert false
       in
       Acond (address self s.loc op, X.Assume (tt n), X.Assume (tf n))
+  | PSinstr (Jl, op) ->
+      Acond (address self s.loc op, X.Assume "cmp < 0", X.Assume "cmp >= 0")
+  | PSinstr (Jle, op) ->
+      Acond (address self s.loc op, X.Assume "cmp <= 0", X.Assume "cmp > 0")
   | PSinstr (Jge, op) ->
       Acond (address self s.loc op, X.Assume "cmp >= 0", X.Assume "cmp < 0")
+  | PSinstr (Jg, op) ->
+      Acond (address self s.loc op, X.Assume "cmp > 0", X.Assume "cmp <= 0")
+  | PSinstr (Je, op) ->
+      Acond (address self s.loc op, X.Assume "cmp = 0", X.Assume "cmp <> 0")
+  | PSinstr (Jne, op) ->
+      Acond (address self s.loc op, X.Assume "cmp <> 0", X.Assume "cmp = 0")
   | PSinstr (Hlt, _) -> 
       Ahalt
   | PSinstr _ -> 
