@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: cc.mli,v 1.20 2006-11-03 12:49:01 marche Exp $ i*)
+(*i $Id: cc.mli,v 1.30 2008/02/05 12:10:49 marche Exp $ i*)
 
 (*s Intermediate CC terms. *)
 
@@ -87,6 +92,9 @@ type proof_term = proof cc_term
 
 type validation = proof cc_term
 
+(* Functional programs. *)
+
+type cc_functional_program = (Loc.position * predicate) cc_term
 
 (*s Sequents and obligations. *)
 
@@ -96,4 +104,25 @@ type context_element =
 
 type sequent = context_element list * predicate
 
-type obligation = Loc.position * string * sequent
+type loc_term = Loc.position * term
+type loc_predicate = Loc.position * predicate
+
+type raw_vc_explain =
+  | VCEexternal of string
+  | VCEabsurd
+  | VCEassert of loc_predicate list
+  | VCEpre of string * Loc.position * loc_predicate list 
+      (*r label and loc for the call, then preconditions *)
+  | VCEpost of loc_predicate
+  | VCEwfrel
+  | VCEvardecr of loc_term
+  | VCEinvinit of string * loc_predicate
+      (*r label for the loop, then loop invariant *)
+  | VCEinvpreserv of string * loc_predicate 
+      (*r label for the loop, then loop invariant *)
+
+(*
+type obligation = Loc.floc * vc_explain * string * sequent
+    (* loc, explanation, id, sequent *) 
+*)
+
