@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: coq.mli,v 1.25 2006-11-03 12:49:01 marche Exp $ i*)
+(*i $Id: coq.mli,v 1.28 2008/02/05 12:10:49 marche Exp $ i*)
 
 open Cc
 open Vcg
@@ -33,6 +38,8 @@ val push_decl : Logic_decl.t -> unit
 
 val push_validation : string -> cc_type -> validation -> unit
 
+val push_program : string -> cc_type -> cc_functional_program -> unit
+
 val push_parameter : string -> cc_type -> unit
 
 val output_file : string -> unit
