@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,18 +27,22 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: dispatcher.mli,v 1.17 2007-07-12 10:55:26 marche Exp $ i*)
+(*i $Id: dispatcher.mli,v 1.25 2008/04/10 13:30:41 hubert Exp $ i*)
 
 open Cc
 
 val push_decl : Logic_decl.t -> unit
 
-val iter : (Loc.position * string * sequent Env.scheme -> unit) -> unit
+val iter : 
+    (Loc.floc * Logic_decl.vc_expl * string * sequent Env.scheme -> unit) 
+    -> unit
 
 type prover = 
   | Simplify | Harvey | Cvcl | Zenon | Rvsat | Yices | Ergo 
   | Cvc3 | Graph | Z3
 
+val prover_name : prover -> string
+
 val call_prover : 
   ?debug:bool -> ?timeout:int -> ?encoding:Options.encoding ->
   obligation:string -> prover -> Calldp.prover_result
