@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,14 +27,14 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: encoding_rec.ml,v 1.8 2006-11-03 12:49:02 marche Exp $ i*)
+(*i $Id: encoding_rec.ml,v 1.12 2008/02/05 12:10:49 marche Exp $ i*)
 
 open Options
 open Cc
 open Logic
 open Logic_decl
 
-let loc = Loc.dummy_position
+let loc = Loc.dummy_floc
 
 let prefix = "c_"
 let suffix = "_c"
@@ -295,7 +300,7 @@ let rec push d =
       Queue.add (Daxiom (loc, ident,
 			 Env.empty_scheme 
  			   (lifted fv (translate_eq [] pred_sch.Env.scheme_type)))) queue
-  | Dgoal (loc, ident, s_sch) ->
+  | Dgoal (loc, expl, ident, s_sch) ->
       let cpt = ref 0 in
       let (cel, pred) = s_sch.Env.scheme_type in
       let fv = Env.Vset.fold
@@ -367,7 +372,7 @@ let rec push d =
 	    lifted q (Forall(false, Ident.create a, Ident.create a, ut, [], p))
       in
       Queue.add (Dgoal 
-		   (loc, ident,
+		   (loc, expl, ident,
 		    Env.empty_scheme 
 		      (List.map 
 			 (fun s -> match s with 
