@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,13 +27,13 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: encoding_strat.ml,v 1.11 2006-12-14 09:29:38 filliatr Exp $ i*)
+(*i $Id: encoding_strat.ml,v 1.16 2008/02/05 12:10:49 marche Exp $ i*)
 
 open Cc
 open Logic
 open Logic_decl
 
-let loc = Loc.dummy_position
+let loc = Loc.dummy_floc
 
 let prefix = "c_"
 let suffix = "_c"
@@ -162,7 +167,8 @@ let rec translate_term fv lv = function
   | Tconst (ConstUnit) as t -> plunge [] t PTunit
   | Tconst (ConstFloat f) as t -> plunge [] t PTreal
   | Tderef id as t -> print_endline ("id in Tderef : "^(Ident.string id)); t
-	
+  | Tnamed(_,t) -> translate_term fv lv t
+
 (* Generalizing a predicate scheme with foralls (can add a trigger) *)
 let rec lifted  l p t =
   match l with [] -> p
@@ -340,7 +346,7 @@ let rec push d =
 	Env.empty_scheme (lifted fv (translate_pred fv [] pred_sch.Env.scheme_type) []) in
       Queue.add (Daxiom (loc, ident, new_axiom)) queue
 (* A goal is a sequent : a context and a predicate and both have to be translated *)
-  | Dgoal (loc, ident, s_sch) ->
+  | Dgoal (loc, expl, ident, s_sch) ->
       let cpt = ref 0 in
       let fv = Env.Vset.fold
 	  (fun tv acc -> cpt := !cpt + 1; (tv.tag, tvar^(string_of_int !cpt))::acc)
@@ -357,7 +363,7 @@ let rec push d =
 	Env.empty_scheme
 	  (lifted_ctxt fv (List.rev new_cel),
 	   translate_eq (translate_pred fv context (snd (s_sch.Env.scheme_type)))) in
-      Queue.add (Dgoal (loc, ident, new_sequent)) queue)
+      Queue.add (Dgoal (loc, expl, ident, new_sequent)) queue)
   with
     Not_found -> 
       Format.eprintf "Exception caught in : %a\n" Util.print_decl d;
