@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: fastwp.ml,v 1.13 2007-07-23 13:18:55 filliatr Exp $ i*)
+(*i $Id: fastwp.ml,v 1.29 2008/04/15 08:12:50 regisgia Exp $ i*)
 
 (*s Fast weakest preconditions *)
 
@@ -35,6 +40,14 @@ open Util
 open Ast
 open Env
 
+let idmap_union m1 m2 =
+  Idmap.fold 
+    (fun x2 v m1 -> 
+      if Idmap.mem x2 m1 
+      then begin assert (Idmap.find x2 m1 = v); m1 end
+      else Idmap.add x2 v m1)
+    m2 m1
+
 module Subst = struct
 
   type t = { 
@@ -57,13 +70,12 @@ module Subst = struct
       all_vars = Idset.add x s.all_vars }
 
   let add_aux x pt s = 
-    Format.eprintf "add_aux %a@." Ident.print x;
     { s with
 	types = Idmap.add x pt s.types;
 	all_vars = Idset.add x s.all_vars }
 
   let frame env ef s =
-    let r,w,_ = Effect.get_repr ef in
+    let r,w,_,_ = Effect.get_repr ef in
     List.fold_left 
       (fun s x -> 
 	 try 
@@ -75,6 +87,13 @@ module Subst = struct
 
   let find x s = Idmap.find x s.current
 
+  let global_names = ref Idset.empty
+
+  let next_away x s =
+    let x' = next_away x (Idset.union !global_names s) in
+    global_names := Idset.add x' !global_names;
+    x'
+
   let fresh x s =
     assert (Idmap.mem x s.types);
     let x' = next_away x s.all_vars in
@@ -84,6 +103,13 @@ module Subst = struct
       types = Idmap.add x' (Idmap.find x s.types) s.types;
       all_vars = Idset.add x' s.all_vars }
 
+  let fresh_pure x pt s =
+    let x' = next_away x s.all_vars in
+    x',
+    { s with 
+      types = Idmap.add x' pt s.types;
+      all_vars = Idset.add x' s.all_vars }
+
   let write x s = let _,s = fresh x s in s
   let writes = List.fold_right write
 
@@ -99,6 +125,11 @@ module Subst = struct
 	     if not (is_at x) then Idmap.add (at_id x l) x' m else m)
 	  s.current s.sigma }
 
+  let add_vars s1 s2 =
+    { s1 with 
+	types = idmap_union s1.types s2.types;
+	all_vars = Idset.union s1.all_vars s2.all_vars } 
+
   (* debug *)
   open Format
   let print fmt s =
@@ -116,14 +147,6 @@ module Subst = struct
 end
 open Subst
 
-let idmap_union m1 m2 =
-  Idmap.fold 
-    (fun x2 v m1 -> 
-      if Idmap.mem x2 m1 
-      then begin assert (Idmap.find x2 m1 = v); m1 end
-      else Idmap.add x2 v m1)
-    m2 m1
-
 let all_quantifiers ((_,s),ee) =
   let s =
     List.fold_left (fun s (_,(_,sx)) -> idmap_union s sx.types) s.types ee 
@@ -139,10 +162,7 @@ let merge s1 s2 =
 	 try 
 	   let x2 = Subst.find x s2 in if x1 != x2 then Idset.add x d else d
 	 with Not_found -> 
-	   Format.eprintf "@[merge avec %a et %a ; pbm avec x=%a@]@." 
-	     Subst.print s1 Subst.print s2 Ident.lprint x;
-	   Format.eprintf "Idmap.mem x s2 = %b@." (Idmap.mem x s2.sigma);
-	   assert false)
+	   d)
       s1.current Idset.empty
   in
   let s12 = 
@@ -160,7 +180,7 @@ let merge s1 s2 =
     d (s12, Ptrue, Ptrue)
 
 let wpforall = pforall ~is_wp:true
-let wpforalls = foralls ~is_wp:true
+let wpforalls = foralls_many ~is_wp:true
 
 let ssubst_in_predicate s p = simplify (tsubst_in_predicate s p)
 
@@ -197,7 +217,11 @@ and wp0 e s =
       (* OK: true
 	 NE: result=t *)
       let t = Subst.term s (unref_term t) in
-      Ptrue, ((tequality v tresult t, s), [])
+      let p = match e.info.t_result_type with
+	| PureType PTunit -> Ptrue
+	| _ -> tequality v tresult t
+      in
+      Ptrue, ((p, s), [])
   | If (e1, e2, e3) ->
       (* OK: ok(e1) /\ (ne(e1,true) => ok(e2)) /\ (ne(e1,false) => ok(e3))
 	 NE: (ne(e1,true) /\ ne(e2,result)) \/ (ne(e1,false) /\ ne(e3,result)) 
@@ -215,10 +239,8 @@ and wp0 e s =
       let ee x = 
 	let ee2,s2 = exn x ee2 s1 and ee3,s3 = exn x ee3 s1 in
 	let s23,r2,r3 = merge s2 s3 in
-	Format.eprintf "s23 = %a@." Subst.print s23;
 	let ee1,s1 = exn x ee1 s in
 	let s',q1,q23 = merge s1 s23 in
-	Format.eprintf "s' = %a@." Subst.print s';
 	pors [wpand ee1 q1; 
 	      wpands [ne1true;ee2;r2;q23]; 
 	      wpands [ne1false;ee3;r3;q23]], s'
@@ -240,18 +262,22 @@ and wp0 e s =
       ok, ((ne, s2), exns e ee)
   | LetIn (x, e1, e2) ->
       let ok1,((ne1,s1),ee1) = wp e1 s in
+      let x',s1 = match e1.info.t_result_type with
+	| PureType pt -> Subst.fresh_pure x pt s1 
+	| _ -> x, s1
+      in
       let ok2,((ne2,s2),ee2) = wp e2 s1 in
       begin match e1.info.t_result_type with
-	| PureType pt as ty1 ->
-	    let ne1x = subst_in_predicate (subst_onev result x) ne1 in
-	    let ok = wpand ok1 (wpforall x ty1 (wpimplies ne1x ok2)) in
-	    let ne = (*exists x ty1*) (wpand ne1x ne2) in
+	| PureType pt ->
+	    let ne1x = subst_in_predicate (subst_onev result x') ne1 in
+	    let subst = subst_in_predicate (subst_onev x x') in
+	    let ok = wpand ok1 (wpimplies ne1x (subst ok2)) in
+	    let ne = wpand ne1x (subst ne2) in
 	    let ee x =
 	      let ee1,sx1 = exn x ee1 s and ee2,sx2 = exn x ee2 s1 in
 	      let s',r1,r2 = merge sx1 sx2 in
 	      por (wpand ee1 r1) (wpands [ne1x; ee2; r2]), s'
 	    in
-	    let s2 = Subst.add_aux x pt s2 in
 	    ok, ((ne, s2), exns e ee)
 	| Arrow _ ->
 	    assert (not (occur_predicate result ne1));
@@ -267,40 +293,26 @@ and wp0 e s =
 	| Ref _ -> 
 	    assert false
       end
-  | AppRef (e, _, k) 
-  | AppTerm (e, _, k) ->
-      let lab = e.info.t_label in
-      let s = Subst.label lab s in
-      let q = optpost_app (asst_app (change_label "" lab)) k.t_post in
-      let ok,(((ne,s'),ee) as nee) = wp e s in
-      assert (not (occur_predicate result ne));
-      let wr s = Subst.writes (Effect.get_writes k.t_effect) s in
-      let nee = match q with
-	| Some (q', qe) -> 
-	    (let s' = wr s' in
-	     wpand ne (Subst.predicate s' q'.a_value), s'),
-	    (let ee x = 
-	       let q' = List.assoc x qe in
-	       let ee,s' = exn x ee s in
-	       let s' = wr s' in
-	       por ee (wpand ne (Subst.predicate s' q'.a_value)), s'
-	     in
-	     exns e ee)
-	| None -> 
-	    nee
-      in
-      ok, nee
-  | Lam (bl, pl, e) ->
-      (* OK: forall bl. pl => ok(e)
-	 NE: forall bl. pl /\ ne(e, result) *)
-      let s = Subst.frame e.info.t_env e.info.t_effect s in
-      let ok,r = wp e s in
-      let qr = all_quantifiers r in
-      let pl = List.map (fun a -> subst_in_predicate s.sigma a.a_value) pl in
-      let q = List.filter (function (_,PureType _) -> true | _ -> false) bl in
-      wpforalls (q @ qr) (wpimplies (wpands pl) ok),
-      ((Ptrue, s), [])
-  | Assertion (al, e1) ->
+  | LetRef (x, e1, e2) ->
+      begin match e1.info.t_result_type with
+	| PureType pt ->
+	    let ok1,((ne1,s1),ee1) = wp e1 s in
+	    let s1 = Subst.add x pt s1 in
+	    let ok2,((ne2,s2),ee2) = wp e2 s1 in
+	    let ne1x = subst_in_predicate (subst_onev result x) ne1 in
+	    let ok = wpand ok1 ((*wpforall x ty1*) (wpimplies ne1x ok2)) in
+	    let ne = (*exists x ty1*) (wpand ne1x ne2) in
+	    let ee x =
+	      let ee1,sx1 = exn x ee1 s and ee2,sx2 = exn x ee2 s1 in
+	      let s',r1,r2 = merge sx1 sx2 in
+	      por (wpand ee1 r1) (wpands [ne1x; ee2; r2]), s'
+	    in
+	    let s2 = Subst.add_aux x pt s2 in
+	    ok, ((ne, s2), exns e ee)
+	| Arrow _ | Ref _ -> 
+	    assert false
+      end
+  | Assertion (k, al, e1) ->
       (* OK: al /\ ok(e1)
 	 NE: al /\ ne(e1,result) *)
       let ok,((ne1,s'),ee1) = wp e1 s in
@@ -329,32 +341,11 @@ and wp0 e s =
 	     wpforall result e1.info.t_result_type (wpimplies ne1 q) ::
 	     List.map2 post_exn ee1 ql)
       in
-      let nee = 
-	let ee x = let ee,sx = exn x ee1 s in wpand ee (List.assoc x ql), sx in
-	(wpand ne1 q, s'), exns e ee
-      in
-      ok, nee
+      let ne = wpand ne1 q, s' in
+      let ee x = let ee,sx = exn x ee1 s in wpand ee (List.assoc x ql), sx in
+      ok, (ne, exns e ee)
   | Label (l, e) ->
       wp e (Subst.label l s)
-  | LetRef (x, e1, e2) ->
-      begin match e1.info.t_result_type with
-	| PureType pt as ty1 ->
-	    let ok1,((ne1,s1),ee1) = wp e1 s in
-	    let s1 = Subst.add x pt s1 in
-	    let ok2,((ne2,s2),ee2) = wp e2 s1 in
-	    let ne1x = subst_in_predicate (subst_onev result x) ne1 in
-	    let ok = wpand ok1 (wpforall x ty1 (wpimplies ne1x ok2)) in
-	    let ne = (*exists x ty1*) (wpand ne1x ne2) in
-	    let ee x =
-	      let ee1,sx1 = exn x ee1 s and ee2,sx2 = exn x ee2 s1 in
-	      let s',r1,r2 = merge sx1 sx2 in
-	      por (wpand ee1 r1) (wpands [ne1x; ee2; r2]), s'
-	    in
-	    let s2 = Subst.add_aux x pt s2 in
-	    ok, ((ne, s2), exns e ee)
-	| Arrow _ | Ref _ -> 
-	    assert false
-      end
   | Var _ -> 
       (* this must be an impure function, thus OK = NE = true *)
       Ptrue, ((Ptrue, s), [])
@@ -365,7 +356,8 @@ and wp0 e s =
       (* OK: I /\ forall w. (I => (ok(e1) /\ (ne(e1,void) => I /\ var<var@)))
 	 N : false
 	 E : e(e1) *)
-      let s0 = Subst.frame e1.info.t_env e1.info.t_effect s in
+      (* TODO: termination *)
+      let s0 = Subst.writes (Effect.get_writes e1.info.t_effect) s in
       let ok1,((ne1,s1),ee1) = wp e1 s0 in
       let ne1void = tsubst_in_predicate (subst_one result tvoid) ne1 in
       let subst_inv s = match inv with
@@ -373,17 +365,23 @@ and wp0 e s =
 	| Some {a_value=i} -> Subst.predicate s i
       in
       let i0 = subst_inv s0 in 
+      let decphi = match var with
+	| None -> Ptrue
+	| Some (loc,phi,_,r) -> 
+	    let id = Util.reg_explanation (Cc.VCEvardecr(loc,phi)) in
+	    Pnamed(id, Papp (r, [Subst.term s1 phi;Subst.term s0 phi], []))
+      in
       let ok = 
-	wpand 
-	  (subst_inv s)
-	  (wpimplies 
-	      i0
-	      (wpand ok1 (wpimplies ne1void (subst_inv s1))))
+	wpands
+	  [Wp.well_founded_rel var;
+	   subst_inv s;
+	   wpimplies i0 
+	     (wpand ok1 (wpimplies ne1void (wpand (subst_inv s1) decphi)))]
       in
       let ee x =
 	let ee,sx = exn x ee1 s0 in wpand i0 ee, sx
       in
-      ok, ((Pfalse, s), exns e ee)
+      ok, ((Pfalse, s1), exns e ee)
   | Raise (id, None) -> 
       (* OK: true  
 	 N : false  
@@ -425,28 +423,92 @@ and wp0 e s =
 	  (fun (ne,s) ((x,v), (_,((nei,si),_))) ->
 	    let e1x,_ = exn x ee1 s in 
 	    let e1x = bind_result v e1x in
-	    let si = 
-	      with_exception_type x v (fun v pt -> Subst.add_aux v pt) si 
-	    in
+	    let si = with_exception_type x v Subst.add_aux si in
 	    let s',r1,r2 = merge s si in
 	    por (wpand ne r1) (wpands [e1x; nei; r2]), s')
 	  (ne1,s1) hl
       in
-      let nee = 
-	[] (*TODO*)
-	  (***
-	let ee x = let ee,sx = exn x ee1 s in wpand ee (List.assoc x ql), sx in
-	(wpand ne1 q, s'), exns e ee
-	  ***)
+      let ee x = 
+	let eex,sx =
+	  if List.exists (fun ((xi,_),_) -> x == xi) hl then
+ 	    Pfalse, s
+	  else
+ 	    exn x ee1 s
+	in
+	List.fold_left
+	  (fun (nex,sx) ((xi,vi),(_,(_,eei))) ->
+	    let e1xi,sxi = exn xi ee1 s in
+	    let eeix,sxi = exn x eei sxi in
+	    let eeix = bind_result vi eeix in
+	    let sxi = with_exception_type xi vi Subst.add_aux sxi in
+	    let sx,r1,r2 = merge sx sxi in
+	    por (wpand nex r1) (wpands [e1xi; eeix; r2]), sx)
+	  (eex, sx) hl
       in
-      ok, (ne, nee)
-(*| Rec of variable * type_v binder list * type_v * variant option * 
-           precondition list * 'a t *)
-  | Rec _ ->
-      assert false (*TODO*)
-(*| Any of type_c *)
-  | Any _ ->
-      assert false (*TODO*)
+      ok, (ne, exns e ee)
+  | Lam (bl, pl, e) ->
+      (* OK: forall bl. pl => ok(e)
+	 NE: forall bl. pl /\ ne(e, result) *)
+      let s = Subst.frame e.info.t_env e.info.t_effect s in
+      let ok,r = wp e s in
+      let qr = all_quantifiers r in
+      let pl = List.map (fun a -> subst_in_predicate s.sigma a.a_value) pl in
+      let q = List.filter (function (_,PureType _) -> true | _ -> false) bl in
+      wpforalls (q @ qr) (wpimplies (wpands pl) ok),
+      ((Ptrue, s), [])
+  | Rec (f, bl, v, var, pl, e) ->
+      (* OK: well_founded(R) /\ forall bl. pl => ok(e)
+	 NE: forall bl. pl /\ ne(e, result) *)
+      let wfr = Wp.well_founded_rel var in
+      let s = Subst.frame e.info.t_env e.info.t_effect s in
+      let ok,r = wp e s in
+      let qr = all_quantifiers r in
+      let pl = List.map (fun a -> subst_in_predicate s.sigma a.a_value) pl in
+      let q = List.filter (function (_,PureType _) -> true | _ -> false) bl in
+      pand wfr (wpforalls (q @ qr) (wpimplies (wpands pl) ok)),
+      ((Ptrue, s), [])
+  | AppRef (e1, _, k) 
+  | AppTerm (e1, _, k) ->
+      let lab = e1.info.t_label in
+      let s = Subst.label lab s in
+      let q = optpost_app (asst_app (change_label "" lab)) k.t_post in
+      let ok,(((ne,s'),ee) as nee) = wp e1 s in
+      assert (not (occur_predicate result ne));
+      let wr s = Subst.writes (Effect.get_writes k.t_effect) s in
+      let nee = match q with
+	| Some (q', qe) -> 
+	    (let s' = wr s' in
+	     wpand ne (Subst.predicate s' q'.a_value), s'),
+	    (let ee x = 
+	       let q' = List.assoc x qe in
+	       let ee,s' = exn x ee s in
+	       let s' = wr s' in
+	       por ee (wpand ne (Subst.predicate s' q'.a_value)), s'
+	     in
+	     exns e ee)
+	| None -> 
+	    nee
+      in
+      ok, nee
+  | Any k ->
+      let lab = e.info.t_label in
+      let s = Subst.label lab s in
+      let q = optpost_app (post_named e.info.t_loc) k.c_post in
+      let q = optpost_app (asst_app (change_label "" lab)) q in
+      let s' = Subst.writes (Effect.get_writes k.c_effect) s in
+      let nee = match q with
+	| Some (q', qe) -> 
+	    (Subst.predicate s' q'.a_value, s'),
+	    (let ee x = 
+	       let q' = List.assoc x qe in
+	       Subst.predicate s' q'.a_value, s'
+	     in
+	     exns e ee)
+	| None -> 
+	    let ee x = Ptrue, s' in
+	    (Ptrue, s'), exns e ee
+      in
+      Ptrue, nee
 
 let wp e =
   let s = Subst.frame e.info.t_env e.info.t_effect Subst.empty in
