@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: hol4.ml,v 1.17 2006-11-24 13:38:12 filliatr Exp $ i*)
+(*i $Id: hol4.ml,v 1.22 2008/02/05 12:10:49 marche Exp $ i*)
 
 (*s HOL 4 output (contributed by Seungkeol Choe, University of Utah) *)
 
@@ -51,7 +56,8 @@ let push_decl = function
   | Daxiom (_, id, p) -> Queue.add (Axiom (id, p)) elem_q
   | Dpredicate_def (_, id, p) -> Queue.add (Predicate (id, p)) elem_q
   | Dfunction_def _ -> assert false (*TODO*)
-  | Dgoal (loc,id,s) -> Queue.add (Obligation (loc,id,s.Env.scheme_type)) elem_q
+  | Dgoal (loc,expl,id,s) -> 
+      Queue.add (Obligation (loc,expl,id,s.Env.scheme_type)) elem_q
   | Dtype _ -> assert false (*TODO*)
 
 (*s Pretty print *)
@@ -152,6 +158,9 @@ let rec print_term fmt = function
   | Tapp (id, tl, _) ->
       fprintf fmt "@[(%a@ %a)@]" 
 	Ident.print id (print_list space print_term) tl
+  | Tnamed (User n, t) ->
+      fprintf fmt "@[(* %s: *) %a@]" n print_term t
+  | Tnamed (_, t) -> print_term fmt t
 
 and print_terms fmt tl = 
   print_list space print_term fmt tl
@@ -213,8 +222,9 @@ let rec print_predicate fmt = function
 	print_pure_type t print_predicate p'
   | Pfpi _ ->
       failwith "fpi not supported in HOL Light"
-  | Pnamed (n, p) ->
+  | Pnamed (User n, p) ->
       fprintf fmt "@[(* %s: *) %a@]" n print_predicate p
+  | Pnamed (_, p) -> print_predicate fmt p
 
 let print_sequent fmt (hyps,concl) =
   let rec print_seq fmt = function
@@ -243,11 +253,13 @@ let print_axiom fmt id v =
   fprintf fmt "(* axiom %s *);;" id
 
 let print_obligation fmt loc id sq =
+(*
   fprintf fmt "@[(* %a *)@]@\n" Loc.report_obligation_position loc;
+*)
   fprintf fmt "val %s = Parse.Term `%a`;;@\n@\n" id print_sequent sq
 
 let print_elem fmt = function
-  | Obligation (loc, s, sq) -> print_obligation fmt loc s sq
+  | Obligation (loc, expl, s, sq) -> print_obligation fmt loc s sq
   | Logic (id, t) -> print_logic fmt id t
   | Axiom (id, p) -> print_axiom fmt id p
   | Predicate _ -> assert false (*TODO*)
@@ -255,7 +267,7 @@ let print_elem fmt = function
 let print_obl_list fmt = 
   let comma = ref false in
   let print = function
-    | Obligation (_,id,_) -> 
+    | Obligation (_,_,id,_) -> 
 	if !comma then fprintf fmt "; "; fprintf fmt "%s" id; comma := true
     | Axiom _ | Logic _ | Predicate _ -> 
 	()
