@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: ident.mli,v 1.54 2006-11-21 22:02:26 filliatr Exp $ i*)
+(*i $Id: ident.mli,v 1.57 2008/04/15 07:50:19 regisgia Exp $ i*)
 
 (*s Identifiers. *)
 
@@ -77,6 +82,18 @@ val exit_exn : t
 
 val is_index : string -> bool
 
+(*s Non termination monad. *)
+
+val non_termination_monad : t
+val nt_unit : t
+val nt_bind : t
+val nt_fix : t
+val nt_lift : t
+
+(*s Identifiers for the functional validation. *)
+
+val fun_id : t -> t
+
 (*s Some pre-defined identifiers. *)
 
 val anonymous : t
@@ -176,6 +193,7 @@ val store : t
 val annot_bool : t
 val well_founded : t
 val well_founded_induction : t
+val wfix : t
 val false_rec : t
 
 val any_int : t
