@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -195,6 +200,9 @@ let rec print_term fmt = function
   | Tapp (id, tl, _) ->
       fprintf fmt "@[(%a@ %a)@]" 
 	ident id (print_list space print_term) tl
+  | Tnamed (User n, t) ->
+      fprintf fmt "@[(* %s: *)@ %a@]" (String.escaped n) print_term t
+  | Tnamed (_, t) -> print_term fmt t
 
 and print_terms fmt tl = 
   print_list space print_term fmt tl
@@ -255,8 +263,9 @@ let rec print_predicate fmt = function
 	print_pure_type t print_predicate p'
   | Pfpi _ ->
       failwith "fpi not supported in Isabelle/HOL"
-  | Pnamed (n, p) ->
+  | Pnamed (User n, p) ->
       fprintf fmt "@[(* %s: *)@ %a@]" (String.escaped n) print_predicate p
+  | Pnamed (_, p) -> print_predicate fmt p
 
 let print_sequent fmt (hyps,concl) =
   let rec print_seq fmt = function
@@ -301,12 +310,15 @@ let reprint_axiom fmt id p =
 let print_axiom fmt id p = 
   reprint_axiom fmt id p
 
-let reprint_obligation fmt loc id s =
+let reprint_obligation fmt loc expl id s =
   fprintf fmt "@[(* %a *)@]@\n" Loc.report_obligation_position loc;
   fprintf fmt "@[<hov 4>(*Why goal*) lemma %a:@\n%a;@]@\n" idents id print_sequent s
+(*;
+  fprintf fmt "@[(* %a *)@]@\n" Util.print_explanation expl
+*)
 
-let print_obligation fmt loc id s = 
-  reprint_obligation fmt loc id s;
+let print_obligation fmt loc expl id s = 
+  reprint_obligation fmt loc expl id s;
   fprintf fmt "(* FILL PROOF HERE *)@\n@\n"
 
 let reprint_predicate fmt id p =
@@ -368,7 +380,9 @@ struct
   let print_element fmt e = 
     begin match e with
       | Parameter _-> assert false
-      | Obligation (loc, id, s) -> print_obligation fmt loc id s.Env.scheme_type
+      | Program _ -> assert false
+      | Obligation (loc, expl, id, s) -> 
+	  print_obligation fmt loc expl id s.Env.scheme_type
       | Logic (id, t) -> print_logic fmt id t
       | Axiom (id, p) -> print_axiom fmt id p
       | Predicate (id, p) -> print_predicate fmt id p
@@ -379,7 +393,9 @@ struct
       
   let reprint_element fmt = function
     | Parameter _ -> assert false
-    | Obligation (loc, id, s) -> reprint_obligation fmt loc id s.Env.scheme_type
+    | Program _ -> assert false
+    | Obligation (loc, expl, id, s) -> 
+	reprint_obligation fmt loc expl id s.Env.scheme_type
     | Logic (id, t) -> reprint_logic fmt id t
     | Axiom (id, p) -> reprint_axiom fmt id p
     | Predicate (id, p) -> reprint_predicate fmt id p
@@ -405,7 +421,7 @@ end)
 let reset = Gen.reset
 
 let push_decl = function
-  | Dgoal (loc,l,s) -> Gen.add_elem (Oblig, l) (Obligation (loc,l,s))
+  | Dgoal (loc,expl,l,s) -> Gen.add_elem (Oblig, l) (Obligation (loc,expl,l,s))
   | Dlogic (_, id, t) -> Gen.add_elem (Lg, rename id) (Logic (id, t))
   | Daxiom (_, id, p) -> Gen.add_elem (Ax, rename id) (Axiom (id, p))
   | Dpredicate_def (_, id, p) -> Gen.add_elem (Pr, rename id) (Predicate (id, p))
