@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,12 +27,10 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(* $Id: lexer.mli,v 1.6 2007-06-15 11:48:43 marche Exp $ *)
+(* $Id: lexer.mli,v 1.9 2008/02/05 12:10:49 marche Exp $ *)
 
 exception Lexical_error of string
 
-val int_is_ident : bool ref
-
 val token : Lexing.lexbuf -> Parser.token
 
 val parse_file  : Lexing.lexbuf -> Ptree.file
