@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: ltyping.ml,v 1.63 2007-03-01 14:40:51 filliatr Exp $ i*)
+(*i $Id: ltyping.ml,v 1.70 2008/04/15 08:12:50 regisgia Exp $ i*)
 
 (*s Typing on the logical side *)
 
@@ -117,22 +122,27 @@ let rec unify t1 t2 = match (t1,t2) with
     (PTunit | PTreal | PTbool | PTint | PTvar {user=true}) -> 
       t1 = t2
 
-let make_comparison loc = function
-  | (a,PTint), (PPlt|PPle|PPgt|PPge|PPeq|PPneq as r), (b,PTint) ->
+let make_comparison loc (a,ta) r (b,tb) = 
+  match normalize_pure_type ta, r, normalize_pure_type tb with
+  | PTint, (PPlt|PPle|PPgt|PPge|PPeq|PPneq), PTint ->
       Papp (int_cmp r, [a; b], [])
-  | (a,PTreal), (PPlt|PPle|PPgt|PPge|PPeq|PPneq as r), (b,PTreal) ->
+  | PTreal, (PPlt|PPle|PPgt|PPge|PPeq|PPneq), PTreal ->
       Papp (real_cmp r, [a; b], [])
-  | (a,ta), r, (b,tb) ->
+  | (PTbool | PTunit as ta), (PPeq|PPneq), (PTbool | PTunit) ->
+      Papp (other_cmp (ta,r), [a; b], [])
+  | ta, (PPeq|PPneq), tb ->
       if unify ta tb then
 	match normalize_pure_type ta, normalize_pure_type tb with
 	  | PTint, PTint -> Papp (int_cmp r, [a;b], [])
 	  | PTreal, PTreal -> Papp (real_cmp r, [a;b], [])
-	  | _ -> Papp (other_cmp (ta,r), [a; b], [])
+	  | _ -> Papp (other_cmp (ta,r), [a; b], [ta])
       else
 	raise_located loc 
 	  (ExpectedType2 
 	     ((fun f -> Util.print_pure_type f (normalize_pure_type ta)),
 	      (fun f -> Util.print_pure_type f (normalize_pure_type tb))))
+  | ta, _, _ ->
+      raise_located loc (IllegalComparison (fun fmt -> print_pure_type fmt ta))
 
 let int_arith = function
   | PPadd -> t_add_int
@@ -219,7 +229,7 @@ and desc_predicate loc lab env = function
       let q = { pp_desc = PPinfix (a, r, b); pp_loc = loc } in
       Pand (false, true, predicate lab env p, predicate lab env q)
   | PPinfix (a, (PPlt | PPle | PPgt | PPge | PPeq | PPneq as r), b) ->
-      make_comparison a.pp_loc (term lab env a, r, term lab env b)
+      make_comparison a.pp_loc (term lab env a) r (term lab env b)
   | PPinfix (_, (PPadd | PPsub | PPmul | PPdiv | PPmod), _) -> 
       predicate_expected loc
   | PPprefix (PPneg, _) ->
@@ -249,7 +259,7 @@ and desc_predicate loc lab env = function
 	 | _ -> raise_located e.pp_loc 
 	         (AnyMessage "this expression should have type real"))
   | PPnamed (n, a) ->
-      Pnamed (n, predicate lab env a)
+      Pnamed (User n, predicate lab env a)
 
 and type_pvar loc env x =
   if is_at x then 
@@ -300,7 +310,10 @@ and desc_term loc lab env = function
 	 | ta, ty when unify ty PTint -> Tapp (t_neg_int, [ta], []), PTint
 	 | ta, ty when unify ty PTreal -> Tapp (t_neg_real, [ta], []), PTreal
 	 | _ -> expected_num loc)
-  | PPprefix (PPnot, _) | PPforall _ | PPexists _ | PPfpi _ | PPnamed _ ->
+  | PPnamed(n, t) -> 
+      let tt,ty = term lab env t in Tnamed(User n, tt), ty
+
+  | PPprefix (PPnot, _) | PPforall _ | PPexists _ | PPfpi _ ->
       term_expected loc
 
 and type_if lab env a b c =
@@ -378,8 +391,10 @@ and pattern lab env t =
 	  PPat (predicate lab env t)
       with Not_found -> 
 	raise_located t.pp_loc (UnboundVariable x))
-  | PPvar x -> TPat (fst (term lab env t))
-  | _ -> Report.raise_located t.pp_loc Error.IllformedPattern
+  | PPvar _ | PPinfix (_, (PPadd|PPsub|PPmul|PPdiv|PPmod), _) ->
+      TPat (fst (term lab env t))
+  | _ -> 
+      Report.raise_located t.pp_loc Error.IllformedPattern
   
 and triggers lab env = List.map (List.map (pattern lab env))
 
@@ -430,11 +445,12 @@ let check_effect loc env e =
   let check_exn id =
     if not (Env.is_exception id) then raise_located loc (UnboundException id)
   in
-  let r,w,x = Effect.get_repr e in
+  let r,w,x,_ = Effect.get_repr e in
   List.iter check_ref r;
   List.iter check_ref w;
   List.iter check_exn x
 
+
 (* warns if a ref occuring in a predicate is not mentioned in the effect,
    and adds it as read to the effect *)
 let warn_refs loc env p = 
