@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: ltyping.mli,v 1.25 2007-03-01 14:40:51 filliatr Exp $ i*)
+(*i $Id: ltyping.mli,v 1.28 2008/04/10 14:43:57 filliatr Exp $ i*)
 
 (*s Typing on the logical side *)
 
@@ -61,6 +66,8 @@ val binders :
   ptype_v binder list -> 
   type_v binder list * local_env
 
+val instance : Ident.t -> var_subst -> pure_type list
+
 (* errors *)
 
 val expected_num : Loc.position -> 'a
