@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: main.ml,v 1.131 2007-06-15 11:48:43 marche Exp $ i*)
+(*i $Id: main.ml,v 1.152 2008/05/28 13:51:26 marche Exp $ i*)
 
 open Options
 open Ptree
@@ -46,7 +51,7 @@ open Logic_decl
 let declarationQueue = Queue.create ()
 
 let reset () =
-  Queue.clear declarationQueue ;
+  (*Queue.clear declarationQueue ;*)
   Vcg.logs := []; 
   match prover () with
     | Pvs -> Pvs.reset ()
@@ -57,7 +62,7 @@ let reset () =
     | Hol4 -> Hol4.reset ()
     | SmtLib ->  ()
     | Harvey | Simplify | Zenon | CVCLite  | Gappa 
-    | Why | MultiWhy | Dispatcher -> ()
+    | Why | MultiWhy | Dispatcher | WhyProject -> ()
 
 let add_loc = function
   | Dtype (loc, _, s)
@@ -74,35 +79,37 @@ let store_decl_into_a_queue d  =
 (** push the declarations  in the corresponding 
     prover and stores them (or their expansed version) into 
     declarationQueue **)
-let push_decl d = 
+let push_decl vloc d = 
   add_loc d;
-  if (not pruning) then
-    let pushing = 
-      match prover () with
-	| Pvs -> Pvs.push_decl
-	| Coq _ -> Coq.push_decl 
-	| HolLight -> Holl.push_decl 
-	| Mizar -> Mizar.push_decl
-	| Isabelle -> Isabelle.push_decl
-	| Hol4 -> Hol4.push_decl
-	| Gappa -> Gappa.push_decl 
-	| Why | MultiWhy -> Pretty.push_decl
-	| Dispatcher ->Dispatcher.push_decl
-	| Harvey -> Harvey.push_decl
-	| Simplify -> Simplify.push_decl
-	| Zenon -> Zenon.push_decl
-	| CVCLite -> Cvcl.push_decl
-	| SmtLib -> Smtlib.push_decl 
-    in
-    if defExpanding != NoExpanding  then 
-      let decl = (PredDefExpansor.push d) in 
-      List.iter pushing decl ;
-      List.iter store_decl_into_a_queue decl ; 
-    else
-      begin 
-	store_decl_into_a_queue d ; 
-	pushing d 
-      end
+  if (not pruning) && (Options.pruning_hyp_v = -1) then
+    begin 
+      let pushing = 
+	match prover () with
+	  | Pvs -> Pvs.push_decl
+	  | Coq _ -> Coq.push_decl 
+	  | HolLight -> Holl.push_decl 
+	  | Mizar -> Mizar.push_decl
+	  | Isabelle -> Isabelle.push_decl
+	  | Hol4 -> Hol4.push_decl
+	  | Gappa -> Gappa.push_decl 
+	  | Why | MultiWhy | WhyProject -> Pretty.push_decl 
+	  | Dispatcher ->Dispatcher.push_decl
+	  | Harvey -> Harvey.push_decl
+	  | Simplify -> Simplify.push_decl
+	  | Zenon -> Zenon.push_decl
+	  | CVCLite -> Cvcl.push_decl
+	  | SmtLib -> Smtlib.push_decl 
+      in
+      if defExpanding != NoExpanding  then 
+	let decl = (PredDefExpansor.push d) in 
+	List.iter pushing decl ;
+	List.iter store_decl_into_a_queue decl ; 
+      else
+	begin 
+	  store_decl_into_a_queue d ;
+	  pushing d 
+	end
+    end
   else
     if defExpanding != NoExpanding then
       List.iter store_decl_into_a_queue (PredDefExpansor.push d)
@@ -111,17 +118,19 @@ let push_decl d =
 	
 	
 	
-let push_obligations = 
+let push_obligations vloc = 
   List.iter 
-    (fun (loc,id,s) -> 
-       let dg = Dgoal (loc, id, generalize_sequent s) in
-       let dg = 
-	 if pruning_hyp != -1 then 
-	   Hypotheses_filtering.reduce dg
-	 else
-	   dg 
-       in
-       push_decl dg) 
+    (fun (loc,expl,id,s) -> 
+      let (loc,expl,id,s) = 
+	if pruning_hyp_v != -1 then 
+	  Hypotheses_filtering.reduce (loc, expl, id, s) declarationQueue
+	else
+	  (loc, expl, id, s)
+      in
+      push_decl vloc (Dgoal(loc, expl, id, generalize_sequent s))
+    ) 
+
+
 
 let prover_is_coq = match prover () with Coq _ -> true | _ -> false
 
@@ -135,7 +144,7 @@ let push_parameter id v tv = match prover () with
       if valid then Coq.push_parameter id tv
   | Pvs | HolLight | Isabelle | Hol4 | Mizar
   | Harvey | Simplify | Zenon | SmtLib | Gappa 
-  | CVCLite | Why | MultiWhy | Dispatcher -> 
+  | CVCLite | Why | MultiWhy | Dispatcher | WhyProject -> 
       ()
 
 let output fwe = 
@@ -161,6 +170,7 @@ let output fwe =
     | Dispatcher -> ()
     | Why -> Pretty.output_file fwe
     | MultiWhy -> Pretty.output_files fwe
+    | WhyProject -> Pretty.output_project fwe
   end
 
 
@@ -170,8 +180,6 @@ let output fwe =
     @q is the queue of  theory that will be modified
 **)
 let encode q = 
-  Printf.printf "Encode"; 
-  
   let callEncoding d =  match prover () with
     | Pvs -> Pvs.push_decl d
     | Coq _ -> Coq.push_decl d
@@ -180,7 +188,7 @@ let encode q =
     | Isabelle -> Isabelle.push_decl d
     | Hol4 -> Hol4.push_decl d
     | Gappa -> Gappa.push_decl d  
-    | Why | MultiWhy -> Pretty.push_decl d
+    | Why | MultiWhy | WhyProject -> Pretty.push_decl d
     | Dispatcher -> Dispatcher.push_decl d      
     | Harvey -> Harvey.push_decl d
     | Simplify -> Simplify.push_decl d
@@ -194,7 +202,7 @@ let encode q =
 
 let print_if_debug p x = if_debug_3 eprintf "  @[%a@]@." p x
 
-let interp_program id p =
+let interp_program loc id p =
   reset_names ();
   let ploc = p.ploc in
   if_verbose_3 eprintf "*** interpreting program %a@." Ident.print id;
@@ -240,20 +248,48 @@ let interp_program id p =
 
   if_debug eprintf "* generating obligations@.";
   let ids = Ident.string id in
+
+  let (name,beh,loc) as vloc =
+    try 
+      let (f,l,b,e,o) = 
+	Hashtbl.find locs_table ids 
+      in 
+      let name = 
+	match List.assoc "name" o with
+	  | Rc.RCident s -> s
+	  | Rc.RCstring s -> s
+	  | _ -> raise Not_found		  
+      in
+      let beh = 
+	match List.assoc "behavior" o with
+	  | Rc.RCstring s -> s
+	  | _ -> raise Not_found		  
+      in
+      (name,beh,(f,l,b,e))
+    with Not_found -> ("function "^ids,"Correctness", Loc.extract loc)
+      
+  in
+  Hashtbl.add program_locs ids vloc;
+
   (*let ol,v = Vcg.vcg ids cc in*)
   begin match wp with
     | None -> 
 	if_debug eprintf "no WP => no obligation@."
     | Some wp -> 
-	let ol,pr = Vcg.vcg_from_wp ids wp in
-	push_obligations ol;
+	if_debug eprintf "VCs from WP...@?";
+	let ol,pr = Vcg.vcg_from_wp loc ids name beh wp in
+	if_debug eprintf "done@.";
+	push_obligations vloc ol;
 	push_validation (ids ^ "_wp") (TTpred wp.a_value) (CC_hole pr)
   end;
 
   if valid then
     let ren = initial_renaming env in
     let tt = Monad.trad_type_c ren env c in
-    Coq.push_parameter (ids ^ "_valid") tt;
+    let ren = initial_renaming env in
+    let cc = Red.red (Mlize.trad p ren) in
+(*    Coq.push_parameter (ids ^ "_valid") tt; *)
+      Coq.push_program (ids ^ "_functional") tt cc;
     (*** TODO
 	 push_validation ids tt v;
 	 if_verbose_2 eprintf "%d proof obligation(s)@\n@." (List.length ol);
@@ -288,12 +324,23 @@ let logic_type_is_var = function
   | Function ([], pt) -> is_a_type_var pt
   | Function _ | Predicate _ -> false
 
+let check_duplicate_params l =
+  let rec loop l acc =
+    match l with
+      | [] -> ()
+      | (loc,x,_)::rem ->
+	  if Ident.Idset.mem x acc then
+	    raise_located loc (ClashParam x)
+	  else loop rem (Ident.Idset.add x acc)
+  in
+  loop l Ident.Idset.empty
+
 let interp_decl ?(prelude=false) d = 
   let lab = Label.empty in
   match d with 
-    | Program (id, p) ->
+    | Program (loc,id, p) ->
 	if Env.is_global id then raise_located p.ploc (Clash id);
-	(try interp_program id p with Exit -> ())
+	(try interp_program loc id p with Exit -> ())
     | Parameter (loc, ext, ids, v) ->
 	let env = Env.empty_progs () in
 	let v = Ltyping.type_v loc lab env v in
@@ -333,24 +380,26 @@ let interp_decl ?(prelude=false) d =
 	    end
 	  else
 	    begin
-	      push_decl (Dlogic (loc, Ident.string id, t));
+	      push_decl ("","",Loc.dummy_floc) (Dlogic (Loc.extract loc, Ident.string id, t));
 	    end 
 	in
 	List.iter add ids
     | Predicate_def (loc, id, pl, p) ->
 	let env = Env.empty_logic () in
 	if is_global_logic id then raise_located loc (Clash id);
-	let pl = List.map (fun (x,t) -> (x, Ltyping.pure_type env t)) pl in
+	check_duplicate_params pl;
+	let pl = List.map (fun (_,x,t) -> (x, Ltyping.pure_type env t)) pl in
 	let t = Predicate (List.map snd pl) in
 	let env' = List.fold_right (fun (x,pt) -> add_logic x pt) pl env in
 	let p = Ltyping.predicate lab env' p in
 	add_global_logic id (generalize_logic_type t);
 	let p = generalize_predicate_def (pl,p) in
-	push_decl (Dpredicate_def (loc, Ident.string id, p))
+	push_decl ("","",Loc.dummy_floc) (Dpredicate_def (Loc.extract loc, Ident.string id, p))
     | Function_def (loc, id, pl, ty, e) ->
 	let env = Env.empty_logic () in
 	if is_global_logic id then raise_located loc (Clash id);
-	let pl = List.map (fun (x,t) -> (x, Ltyping.pure_type env t)) pl in
+	check_duplicate_params pl;
+	let pl = List.map (fun (_,x,t) -> (x, Ltyping.pure_type env t)) pl in
 	let ty = Ltyping.pure_type env ty in
 	let t = Function (List.map snd pl, ty) in
 	let env' = List.fold_right (fun (x,pt) -> add_logic x pt) pl env in
@@ -359,33 +408,58 @@ let interp_decl ?(prelude=false) d =
 	  Ltyping.expected_type loc (PureType ty);
 	add_global_logic id (generalize_logic_type t);
 	let f = generalize_function_def (pl,ty,e) in
-	push_decl (Dfunction_def (loc, Ident.string id, f))
+	push_decl ("","",Loc.dummy_floc) (Dfunction_def (Loc.extract loc, Ident.string id, f))
     | Axiom (loc, id, p) ->
 	let env = Env.empty_logic () in
 	let p = Ltyping.predicate lab env p in
 	let p = generalize_predicate p in
-	push_decl (Daxiom (loc, Ident.string id, p))
+	push_decl ("","",Loc.dummy_floc) (Daxiom (Loc.extract loc, Ident.string id, p))
+
     | Goal (loc, id, p) ->
 	let env = Env.empty_logic () in
 	let p = Ltyping.predicate lab env p in
-	let s = generalize_sequent ([], p) in
-	let dg = Dgoal (loc, Ident.string id, s) in
-	if Options.pruning_hyp != -1 then
-	  begin
-	    push_decl (Hypotheses_filtering.reduce dg)
-	  end
-	else	  
-	  begin
-	    push_decl (dg)
-	  end
+	let s = ([], p) in
+	let l = Loc.extract loc in
+	let xpl =
+	  { lemma_or_fun_name = Ident.string id;
+	    behavior = "";
+	    vc_loc = l;
+	    vc_kind = EKLemma;
+	  }
+	in
+	let (l,xpl,id,s) = 
+	  if Options.pruning_hyp_v != -1 then
+	    (Hypotheses_filtering.reduce (l, xpl, Ident.string id, s) declarationQueue)
+	  else	  
+	    (l, xpl, Ident.string id, s) 
+	in
+	let s= generalize_sequent s in 
+	let dg = 
+	  Dgoal (l, xpl, id, s) 
+	in
+	let ids = id in
+	let vloc =
+	  try 
+	    let (f,l,b,e,o) = Hashtbl.find locs_table ids in 
+	    let name = 
+	      match List.assoc "name" o with
+		| Rc.RCident s -> s
+		| Rc.RCstring s -> s
+		| _ -> raise Not_found		  
+	    in
+	    (name,"Validity",(f,l,b,e))
+	  with Not_found -> ("goal "^ids,"Validity", Loc.extract loc)
+	in
+	Hashtbl.add program_locs ids vloc;
+	push_decl ("","",Loc.dummy_floc) dg
+	  
+
     | TypeDecl (loc, ext, vl, id) ->
 	Env.add_type loc vl id;
 	let vl = List.map Ident.string vl in
 	if not ext then 
-	  begin
-	    push_decl (Dtype (loc, vl, Ident.string id))
-	  end
-
+	  push_decl ("","",Loc.dummy_floc) (Dtype (Loc.extract loc, vl, Ident.string id))
+	    
 (*s Prelude *)
 
 let load_file ?(prelude=false) f =
@@ -418,7 +492,7 @@ let deal_channel parsef cin =
 
 let single_file () = match prover () with
   | Simplify | Harvey | Zenon | CVCLite | Gappa | Dispatcher 
-  | SmtLib | Why | MultiWhy -> true
+  | SmtLib | Why | MultiWhy | WhyProject -> true
   | Coq _ | Pvs | Mizar | Hol4 | HolLight | Isabelle -> false
 
 let deal_file f =
@@ -432,14 +506,12 @@ let deal_file f =
 let main () =
   let t0 = Unix.times () in
   load_prelude ();
-  (* after prelude because it contains `int' not `integer' *)
-  Lexer.int_is_ident := Options.int_is_ident;
   if files = [] then begin
     deal_channel (why_parser "standard input") stdin;
     output (Options.out_file "out")
   end else begin
     List.iter deal_file files;
-    if pruning then
+    if (pruning) or (Options.pruning_hyp_v != -1) then
       begin
 	let q =  declarationQueue in 
 	encode q ;
