@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: monomorph.ml,v 1.29 2006-11-24 13:28:00 filliatr Exp $ i*)
+(*i $Id: monomorph.ml,v 1.33 2008/02/05 12:10:49 marche Exp $ i*)
 
 (* monomorphic output *)
 
@@ -281,6 +286,7 @@ module OpenInstances = struct
   let rec term s = function
     | Tvar _ | Tderef _ | Tconst _ -> s
     | Tapp (id, l, i) -> List.fold_left term (add (id,i) s) l
+    | Tnamed(_,t) -> term s t
 
   let rec pattern s = function
     | TPat t -> term s t
@@ -528,7 +534,7 @@ let instantiate_axioms loc =
     
 let new_type = let r = ref 0 in fun () -> incr r; "type_" ^ string_of_int !r
 
-let push_obligation loc o s = 
+let push_obligation loc expl o s = 
   let vs, s = specialize_sequent s in
   Vmap.iter
     (fun _ tv -> 
@@ -538,7 +544,7 @@ let push_obligation loc o s =
       vs;
   IterIT.sequent (declare_logic loc) (declare_type loc) s;
   instantiate_axioms loc;
-  push (Dgoal (loc, o, empty_scheme s))
+  push (Dgoal (loc, expl, o, empty_scheme s))
 
 let push_decl = function
   | Dtype (loc, [], id) -> declare_type loc (PTexternal ([], Ident.create id))
@@ -547,7 +553,7 @@ let push_decl = function
   | Dpredicate_def (loc, x, d) -> push_predicate_def loc x d
   | Dfunction_def (loc, x, d) -> push_function_def loc x d
   | Daxiom (loc, x, p) -> push_axiom loc x p
-  | Dgoal (loc, x, s) -> push_obligation loc x s
+  | Dgoal (loc, expl, x, s) -> push_obligation loc expl x s
 
 let reset () =
   Queue.clear queue;
