@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: options.ml,v 1.96 2007-07-23 13:18:55 filliatr Exp $ i*)
+(*i $Id: options.ml,v 1.116 2008-05-25 12:11:28 stoulsn Exp $ i*)
 
 open Format
 
@@ -56,14 +61,31 @@ let all_vc_ = ref false
 let prelude_ = ref true
 let arrays_ = ref true
 let floats_ = ref false
+let eval_goals_ = ref false 
 let pruning_ = ref false 
-let pruning_hyp_ = {contents = -1}
+let pruning_hyp_v_ = {contents = -1}
+let pruning_hyp_p_ = {contents = -1}
+(* Heuristiques en test *)
+let pruning_hyp_CompInGraph_ = ref false
+let pruning_hyp_CompInFiltering_ = ref false
+let pruning_hyp_LinkVarsQuantif_ = ref false
+let pruning_hyp_keep_single_comparison_representation_ = ref true
+let pruning_hyp_comparison_eqOnly_ = ref false
+let pruning_hyp_suffixed_comparison_ = ref false
+let pruning_hyp_equalities_linked_ = ref false
+let pruning_hyp_arithmetic_tactic_ = ref false
+let pruning_hyp_var_tactic_ = ref 0
+let pruning_hyp_polarized_preds_ = ref false
+let prune_context_ = ref false
+(* FIN de Heuristiques en test *)
 let modulo_ = ref false 
 let gappa_rnd_ = ref "float < ieee_64, ne >"
 let lib_files_to_load_ = ref []
 let files_to_load_ = ref []
 let show_time_ = ref false
-let int_is_ident = ref  false
+let locs_files = ref []
+let explain_vc = ref false
+let locs_table = Hashtbl.create 97
 
 type encoding = 
   | NoEncoding | Predicates | Stratified | Recursive | Monomorph 
@@ -94,6 +116,7 @@ let coq_version = match Version.coqversion with "v8" -> V8 | _ -> V7
 type prover = 
   | Coq of coq_version | Pvs | HolLight | Mizar | Harvey | Simplify | CVCLite
   | SmtLib | Isabelle | Hol4 | Gappa | Zenon | Why | MultiWhy | Dispatcher
+  | WhyProject
 
 let prover_ = ref (Coq coq_version)
 
@@ -170,20 +193,38 @@ Typing/Annotations/VCG options:
   -p,  --parse-only  exits after parsing
   -tc, --type-only   exits after type-checking
   -wp, --wp-only     exits after annotation
+  --fast-wp          use WP quadratic algorithm 
   --white            white boxes: WP calculus enters pure expressions
   --black            black boxes: WP calculus does not enter pure expressions
   --wbb              while loops as black boxes (careful: incomplete WP)
   --split-user-conj  splits also user conjunctions in goals 
   --split n          splits conditions into several pieces up to n levels
-  --all-vc           outputs all verification conditions (no auto discharge)
   --partial          partial correctness
   --total            total correctness
+  --explain          outputs explanations for VCs in file.xpl
+  --locs f           reads source locations from file f
+
+VC transformation options:
+  --all-vc           outputs all verification conditions (no auto discharge)
+  --eval-goals       evaluate constant expressions in goals
   --prune-theory     prunes the theory 
-  --prune-hyp k      prunes the hypotheses according to the depth k  
-  --modulo           displays mod in smtlib instead of pourcent
+  --prune-hyp P V    prunes the hypotheses according to the depths P and V  
   --exp all          expands the predicate definitions in both theory and goal 
   --exp goal         expands the predicate definitions only in goal 
 
+Heuristics UNDER TEST of pruning (needs --prun-hyp to be used) :
+  --prune-with-comp           uses comparisons as predicates
+  --prune-with-comp-filter    as previous and uses also comparisons as filters
+  --prune-keep-local-links    insert quantified variables in variables graph
+  --prune-comp-dual-encoding  do not inverse negative comparisons
+  --prune-eq-only             consider only equality comparison as a predicate
+  --prune-suffixed-comp       suffixes comparison predicates
+  --prune-link-eqs            link each suffixed equalitie to the unsuffixed
+  --prune-arith-tactic        statically link arithmetic operators (=, < & <=)
+  --prune-vars-filter T       T in {All, One-var, One-branch, Split-hyps, CNF}        
+  --prune-polarized-preds     Consider polarity on predicates 
+  --prune-context             Filter axioms with Pred depth
+
 Prelude files:
   --no-prelude   do not read the prelude files (prelude.why and arrays.why)
   --no-arrays    do not read the arrays prelude file (arrays.why)
@@ -198,7 +239,6 @@ Encoding for types in untyped logic:
   --encoding mono    encodes types using monomorphisation
   --encoding sstrat  encodes types using some types  + strat
 
-
 Prover selection:
   --coq       selects COQ prover (default)
   --pvs       selects PVS prover
@@ -215,6 +255,7 @@ Prover selection:
 
   --why       selects the Why pretty-printer
   --multi-why selects the Why pretty-printer, with one file per goal
+  --project   selects the Why project format, with one file per goal
 
 Coq-specific options:
   --valid, 
@@ -226,6 +267,9 @@ Coq-specific options:
   --coq-fp-model <Module>
               sets the Coq model for floating-point arithmetic
 
+SMT-lib-specific options:
+  --modulo           uses %% in SMT-lib output (instead of uninterpreted symb)
+
 PVS-specific options:
   --pvs-preamble <text>
               gives some PVS preamble to be substituted to \"importing why@@why\"
@@ -286,6 +330,7 @@ let files =
     | ("-zenon" | "--zenon") :: args -> prover_ := Zenon; parse args
     | ("-why" | "--why") :: args -> prover_ := Why; parse args
     | ("-multi-why" | "--multi-why") :: args -> prover_ := MultiWhy; parse args
+    | ("-project" | "--project") :: args -> prover_ := WhyProject; parse args
     | ("-gappa" | "--gappa") :: args -> prover_ := Gappa; parse args
     | ("-fp" | "--fp") :: args -> floats_ := true; parse args
     | ("-show-time" | "--show-time") :: args -> show_time_ := true; parse args
@@ -390,10 +435,63 @@ let files =
 	termination_ := Partial; parse args
     | ("-total" | "--total") :: args ->
 	termination_ := Total; parse args
+    | ("--eval-goals" | "-eval-goals") :: args ->
+	 eval_goals_ := true ; parse args
     | ("--prune-theory" | "-prune-theory") :: args ->
 	 pruning_ := true ; parse args
-    | ("--prune-hyp" | "-prune-hyp"):: t :: args ->
-	 pruning_hyp_ := int_of_string t ; parse args
+    | ("--prune-hyp" | "-prune-hyp"):: tp :: tv :: args ->
+	pruning_hyp_p_ := int_of_string tp ; 
+	pruning_hyp_v_ := int_of_string tv ; 
+	parse args
+(* Heuristiques en test *)
+    | ("--prune-with-comp" | "-prune-with-comp"):: args ->
+	pruning_hyp_CompInGraph_ := true ; 
+	parse args
+    | ("--prune-with-comp-filter" | "-prune-with-comp-filter"):: args ->
+	pruning_hyp_CompInGraph_ := true ; 
+	pruning_hyp_CompInFiltering_ := true ;
+	parse args
+    | ("--prune-keep-local-links" | "-prune-keep-local-links"):: args ->
+	pruning_hyp_LinkVarsQuantif_ := true ; 
+	parse args
+    | ("--prune-comp-dual-encoding" | "-prune-comp-dual-encoding"):: args ->
+	pruning_hyp_keep_single_comparison_representation_ := false ; 
+	parse args
+    | ("--prune-eq-only" | "-prune-eq-only"):: args ->
+	pruning_hyp_CompInGraph_ := true ; 
+	pruning_hyp_comparison_eqOnly_ := true ; 
+	parse args
+    | ("--prune-suffixed-comp" | "-prune-suffixed-comp"):: args ->
+	pruning_hyp_CompInGraph_ := true ; 
+	pruning_hyp_suffixed_comparison_ := true ; 
+	parse args
+    | ("--prune-link-eqs" | "-prune-link-eqs"):: args ->
+	pruning_hyp_CompInGraph_ := true ; 
+	pruning_hyp_suffixed_comparison_ := true ; 
+	pruning_hyp_equalities_linked_ := true ; 
+	parse args
+    | ("--prune-arith-tactic" | "-prune-arith-tactic"):: args ->
+	pruning_hyp_CompInGraph_ := true ; 
+	pruning_hyp_arithmetic_tactic_ := true ; 
+	parse args
+
+    | ("--prune-vars-filter" | "-prune-vars-filter"):: t :: args ->
+	(match t with 
+	  "All" -> pruning_hyp_var_tactic_:=0; parse args
+	| "One-var" -> pruning_hyp_var_tactic_:=1; parse args 
+	| "One-branch" -> pruning_hyp_var_tactic_:=2; parse args
+	| "Split-hyps" -> pruning_hyp_var_tactic_:=3; parse args
+	| "CNF" ->  pruning_hyp_var_tactic_:=4; parse args
+	| _ -> usage (); exit 1);
+
+    | ("--prune-polarized-preds" | "-prune-polarized-preds"):: args ->
+	pruning_hyp_polarized_preds_ := true ; parse args
+
+    | ("--prune-context" | "-prune-context") :: args ->
+	prune_context_ := true ; parse args
+
+(* FIN de Heuristiques en test *)
+
     | ("-modulo" | "--modulo") :: args ->
 	 modulo_ := true ; parse args
     | ("-exp" | "--exp") :: s :: args ->
@@ -412,8 +510,10 @@ let files =
 	| "sstrat" -> types_encoding_ := SortedStratified
 	| _ -> usage (); exit 1);
 	parse args
-    | "--int-is-ident" :: args ->
-	int_is_ident := true;
+    | ("-explain" | "--explain") :: args ->
+	explain_vc := true; parse args
+    | ("-locs" | "--locs") :: s :: args ->
+	locs_files := s :: !locs_files;
 	parse args
     | f :: args -> 
 	filesq := f :: !filesq; parse args
@@ -461,10 +561,26 @@ let lvlmax = !lvlmax_
 let all_vc = !all_vc_
 let termination = !termination_
 let gappa_rnd = !gappa_rnd_
+let eval_goals = !eval_goals_
 let pruning = !pruning_
-let pruning_hyp = !pruning_hyp_
+let pruning_hyp_p = !pruning_hyp_p_
+let pruning_hyp_v = !pruning_hyp_v_
+(* Heuristiques en test *)
+let pruning_hyp_CompInGraph = !pruning_hyp_CompInGraph_
+let pruning_hyp_CompInFiltering = !pruning_hyp_CompInFiltering_
+let pruning_hyp_LinkVarsQuantif = !pruning_hyp_LinkVarsQuantif_
+let pruning_hyp_keep_single_comparison_representation = !pruning_hyp_keep_single_comparison_representation_
+let pruning_hyp_comparison_eqOnly = !pruning_hyp_comparison_eqOnly_
+let pruning_hyp_suffixed_comparison = !pruning_hyp_suffixed_comparison_
+let pruning_hyp_equalities_linked = !pruning_hyp_equalities_linked_
+let pruning_hyp_arithmetic_tactic = !pruning_hyp_arithmetic_tactic_
+let pruning_hyp_var_tactic = !pruning_hyp_var_tactic_
+let pruning_hyp_polarized_preds = !pruning_hyp_polarized_preds_
+let prune_context = !prune_context_
+(* FIN de Heuristiques en test *)
 let modulo = !modulo_
 let defExpanding = !defExpanding_
+let explain_vc = !explain_vc
 
 (* encoding checks *)
 let () = 
@@ -498,7 +614,26 @@ let if_debug f x = if debug then f x
 let if_debug_2 f x y = if debug then f x y
 let if_debug_3 f x y z = if debug then f x y z
 
-let int_is_ident = !int_is_ident
+let () =
+  List.iter
+    (fun f -> 
+       let l = Rc.from_file f in
+       List.iter
+	 (fun (id,fs) ->
+	    let (f,l,b,e,o) =
+	      List.fold_left
+		(fun (f,l,b,e,o) v ->
+		   match v with
+		     | "file", Rc.RCstring f -> (f,l,b,e,o)
+		     | "line", Rc.RCint l -> (f,l,b,e,o)
+		     | "begin", Rc.RCint b -> (f,l,b,e,o)
+		     | "end", Rc.RCint e -> (f,l,b,e,o)
+		     | _ -> (f,l,b,e,v::o))
+		("",0,0,0,[]) fs
+	    in
+	    Hashtbl.add locs_table id (f,l,b,e,o))
+	 l)
+    !locs_files
 
 (* compatibility checks *)
 let () = if prover () = Gappa && valid then begin
