@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: options.mli,v 1.74 2007-06-15 11:48:43 marche Exp $ i*)
+(*i $Id: options.mli,v 1.88 2008-05-25 12:11:28 stoulsn Exp $ i*)
 
 (*s General options *)
 
@@ -42,7 +47,10 @@ val ocaml : bool
 val ocaml_annot : bool
 val ocaml_externals : bool
 
-val int_is_ident : bool
+val explain_vc : bool
+val locs_table : 
+  (string, (string * int * int * int * (string * Rc.rc_value) list)) 
+     Hashtbl.t
 
 val wol : bool
 
@@ -61,8 +69,23 @@ val wbb : bool
 val split_user_conj : bool
 val lvlmax : int
 val all_vc : bool
+val eval_goals : bool
 val pruning : bool
-val pruning_hyp : int
+val pruning_hyp_v : int
+val pruning_hyp_p : int
+(* Heuristiques en test *)
+val pruning_hyp_CompInGraph : bool
+val pruning_hyp_CompInFiltering : bool
+val pruning_hyp_LinkVarsQuantif : bool
+val pruning_hyp_keep_single_comparison_representation : bool
+val pruning_hyp_comparison_eqOnly : bool
+val pruning_hyp_suffixed_comparison : bool
+val pruning_hyp_equalities_linked : bool
+val pruning_hyp_arithmetic_tactic : bool
+val pruning_hyp_var_tactic : int
+val pruning_hyp_polarized_preds : bool
+val prune_context : bool
+(* FIN de Heuristiques en test *)
 val modulo : bool
 
 type expanding = All | Goal | NoExpanding
@@ -85,6 +108,7 @@ type coq_version = V7 | V8
 type prover = 
   | Coq of coq_version | Pvs | HolLight | Mizar | Harvey | Simplify | CVCLite
   | SmtLib | Isabelle | Hol4 | Gappa | Zenon | Why | MultiWhy | Dispatcher
+  | WhyProject
 
 val prover : unit -> prover
 
@@ -117,6 +141,10 @@ val file : string -> string
 
 val out_file : string -> string
 
+(* [lib_file f] appends [f] to the lib directory *)
+
+val lib_file : string -> string
+
 (*s Files given on the command line *)
 
 val files : string list
