@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: pp.ml,v 1.9 2007-07-06 07:45:28 marche Exp $ i*)
+(*i $Id: pp.ml,v 1.15 2008/02/26 10:16:36 moy Exp $ i*)
 
 (*s Pretty-print library *)
 
@@ -45,26 +50,54 @@ let print_list_or_default default sep print fmt = function
   | [] -> fprintf fmt "%s" default
   | l -> print_list sep print fmt l
 
+let print_list_par sep pr fmt l =
+  print_list sep (fun fmt x -> fprintf fmt "(%a)" pr x) fmt l
+
+let print_list_delim start stop sep pr fmt = function
+  | [] -> ()
+  | l -> fprintf fmt "%a%a%a" start () (print_list sep pr) l stop ()
+
 let comma fmt () = fprintf fmt ",@ "
 let underscore fmt () = fprintf fmt "_"
 let semi fmt () = fprintf fmt ";@ "
-let space fmt () = fprintf fmt "@ "
+let space fmt () = fprintf fmt " "
 let alt fmt () = fprintf fmt "|@ "
 let newline fmt () = fprintf fmt "@\n"
 let arrow fmt () = fprintf fmt "@ -> "
+let lbrace fmt () = fprintf fmt "{"
+let rbrace fmt () = fprintf fmt "}"
 let nothing fmt () = ()
 let string fmt s = fprintf fmt "%s" s
 
 let hov n fmt f x = pp_open_hovbox fmt n; f x; pp_close_box fmt ()
 
-let print_in_file_no_close ?(margin=78) p f =
-  let cout = open_out f in
+let open_formatter ?(margin=78) cout =
   let fmt = formatter_of_out_channel cout in
   pp_set_margin fmt margin;
-  pp_open_box fmt 0; p fmt; pp_close_box fmt ();
-  pp_print_flush fmt ();
+  pp_open_box fmt 0; 
+  fmt
+
+let close_formatter fmt = 
+  pp_close_box fmt ();
+  pp_print_flush fmt ()
+
+let open_file_and_formatter ?(margin=78) f =
+  let cout = open_out f in
+  let fmt = open_formatter ~margin cout in
+  cout,fmt
+
+let close_file_and_formatter (cout,fmt) =
+  close_formatter fmt;
+  close_out cout
+
+let print_in_file_no_close ?(margin=78) p f =
+  let cout,fmt = open_file_and_formatter ~margin f in
+  p fmt;
+  close_formatter fmt;
   cout
 
 let print_in_file ?(margin=78) p f =
   let cout = print_in_file_no_close ~margin p f in
   close_out cout
+
+
