@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: pp.mli,v 1.9 2007-07-06 07:45:28 marche Exp $ i*)
+(*i $Id: pp.mli,v 1.15 2008/02/26 10:16:36 moy Exp $ i*)
 
 open Format
 
@@ -35,6 +40,14 @@ val print_list :
 val print_list_or_default :
   string -> (formatter -> unit -> unit) -> 
   (formatter -> 'a -> unit) -> formatter -> 'a list -> unit
+val print_list_par :
+  (Format.formatter -> unit -> 'a) ->
+  (Format.formatter -> 'b -> unit) -> Format.formatter -> 'b list -> unit
+val print_list_delim :
+  (Format.formatter -> unit -> unit) ->
+  (Format.formatter -> unit -> unit) ->
+  (Format.formatter -> unit -> unit) ->
+  (Format.formatter -> 'b -> unit) -> Format.formatter -> 'b list -> unit
 val space : formatter -> unit -> unit
 val alt : formatter -> unit -> unit
 val newline : formatter -> unit -> unit
@@ -42,8 +55,15 @@ val comma : formatter -> unit -> unit
 val semi : formatter -> unit -> unit
 val underscore : formatter -> unit -> unit
 val arrow : formatter -> unit -> unit
+val lbrace : formatter -> unit -> unit
+val rbrace : formatter -> unit -> unit
 val nothing : formatter -> unit -> unit
+val string : formatter -> string -> unit
 val hov : int -> formatter -> ('a -> unit) -> 'a -> unit
 
+val open_formatter : ?margin:int -> out_channel -> formatter
+val close_formatter : formatter -> unit
+val open_file_and_formatter : ?margin:int -> string -> out_channel * formatter
+val close_file_and_formatter : out_channel * formatter -> unit
 val print_in_file_no_close : ?margin:int -> (Format.formatter -> unit) -> string -> out_channel
 val print_in_file : ?margin:int -> (Format.formatter -> unit) -> string -> unit
