@@ -1,13 +1,18 @@
 (**************************************************************************)
 (*                                                                        *)
-(*  The Why/Caduceus/Krakatoa tool suite for program certification        *)
-(*  Copyright (C) 2002-2006                                               *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
 (*    Jean-Franois COUCHOT                                               *)
 (*    Mehdi DOGGUY                                                        *)
 (*    Jean-Christophe FILLITRE                                           *)
 (*    Thierry HUBERT                                                      *)
 (*    Claude MARCH                                                       *)
 (*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU General Public                   *)
@@ -22,7 +27,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: pretty.mli,v 1.7 2007-02-15 15:52:43 filliatr Exp $ i*)
+(*i $Id: pretty.mli,v 1.10 2008/02/18 09:10:04 marche Exp $ i*)
 
 (* Why pretty-printer *)
 
@@ -35,3 +40,8 @@ val output_file : string -> unit
 (* [output_files f] produces the context in file [f_ctx.why]
    and each goal in a seaparate file [f_po<i>.why] for i=1,2,... *)
 val output_files : string -> unit
+
+(* [output_project f] produces a whole project description, in a file
+[f.wpr], together with other needed files [f_ctx.why], [f_lemmas.why],
+and each goal in a seaparate file [f_po<i>.why] for i=1,2,... *)
+val output_project : string -> unit
